# ##### BEGIN GPL LICENSE BLOCK #####
#
#  This program is free software; you can redistribute it and/or
#  modify it under the terms of the GNU General Public License
#  as published by the Free Software Foundation; either version 2
#  of the License, or (at your option) any later version.
#
#  This program is distributed in the hope that it will be useful,
#  but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#  GNU General Public License for more details.
#
#  You should have received a copy of the GNU General Public License
#  along with this program; if not, write to the Free Software Foundation,
#  Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
#
# ##### END GPL LICENSE BLOCK #####

import copy
from math import *
import os
import platform
from random import random
import struct
import subprocess
import sys
import time

from TheaForBlender.thea_render_main import *
import bpy

from . import thea_globals


class RENDER_PT_thea_startSDK(bpy.types.Operator):
    bl_idname = "thea.start_sdk"
    bl_label = "Start SDK"    
    _handle = None
    
    def modal(self, context, event):                    
        return {'PASS_THROUGH'}
    
    def cancel(self, context):
            print("Killing process with pid %s " % context.scene['thea_sdkProcess'])
            os.kill(context.scene['thea_sdkProcess'],1)
            return {'CANCELLED'}
    
    def invoke(self, context, event):
        if context.scene.thea_sdkIsRunning == False:
            try:
                if context.scene.get('thea_SDKPort'):
                    port = context.scene.get('thea_SDKPort')
                else:
                    port = 30000
            except:
                port = 30000
    
            (exportPath, theaPath, theaDir, dataPath, currentBlendDir, currentBlendFile) = setPaths(context.scene)        
            pArgs = []
            #pArgs.append(os.path.join(theaPath))
            pArgs.append(os.path.join(os.path.dirname(os.path.realpath(__file__)), "TheaRemoteDarkroom.exe"))
            pArgs.append('-remoteserver')
            pArgs.append(str(port))
            pArgs.append('-hidden')
            pArgs.append('-nosplash')
            print(pArgs)
            # start SDK project on preview port
            sdkProcess = subprocess.Popen(pArgs)
            context.scene['thea_sdkProcess'] = sdkProcess.pid
            print("Started process with pid %s " % sdkProcess.pid)
            self.region = context.region
            context.scene.thea_sdkIsRunning = True
            context.window_manager.modal_handler_add(self)                                       
            self._handle = context.region.callback_add(InitGLOverlay, (self, context, self.region, area), 'POST_PIXEL')
            
            print("Starting SDK on port: ", port)            
            print("Started SDK")
            return {'RUNNING_MODAL'}                
        else:
            # operator is called again, stop displaying
            try:
                os.kill(context.scene['thea_sdkProcess'],1)
            except:
                pass
            print("Killing process with pid %s " % context.scene['thea_sdkProcess'])
            context.scene.thea_sdkIsRunning = False
            return {'FINISHED'}

class RENDER_PT_thea_stopSDK(bpy.types.Operator):
    bl_idname = "thea.stop_sdk"
    bl_label = "Start SDK"    
    
    def invoke(self, context, event):
        if bpy.context.scene.sdkProcess.poll() is None:
            print("Killing process with pid %s " % sdkProcess.pid)
            try:
                bpy.context.scene.sdkProcess.kill()
            except:
                pass
        return {'FINISHED'}


def register():
    return

def exportIRCamera(scene, area, exportPath):
    print("exportIRCamera")
    resolutionDivider = 1
    
    cam = ThCamera()    
    camData = area.spaces.active
    if scene.get('thea_IRResolution'):
        if scene.get('thea_IRResolution') == 0:
            resolutionDivider = 1
        if scene.get('thea_IRResolution') == 1:
            resolutionDivider = 2
        if scene.get('thea_IRResolution') == 2:
            resolutionDivider = 4
    
    thea_globals.frame_px = view3d_camera_border(bpy.context.scene, area)
    if thea_globals.frame_px:
        camOb = scene.camera
        cam.name=camOb.name
        camM = scene.camera.matrix_world
        renderData = scene.render
        x1 = thea_globals.frame_px[3][0]
        y2 = thea_globals.frame_px[3][1]
        x2 = thea_globals.frame_px[0][0]
        y1 = thea_globals.frame_px[1][1]       
        resX = int(x2-x1)
        resY = int(y2-y1)
        #resX=int(renderData.resolution_x*renderData.resolution_percentage*0.01)
        #resY=int(renderData.resolution_y*renderData.resolution_percentage*0.01)
        camData = camOb.data
        cam.pixels = resX
        cam.lines = resY    
        
        if resX > resY:
            fac = float(resX) / float(resY)
        else:
            fac = 1
    
        if camData.type == "PERSP":
            cam.focalLength = camData.lens
            cam.focusDistance = 1
            cam.filmHeight = 32 / fac
            cam.shiftLensX = camData.shift_x
            cam.shiftLensY = camData.shift_y
        elif camData.type == "ORTHO":
            cam.projection = "Parallel"
            cam.filmHeight = (camOb.scale*1000)/fac
        if camData.dof_distance > 0:
            cam.focusDistance = camData.dof_distance
            try:
                aperture = camOb["aperture"]
                cam.fNumber = aperture
            except: pass
        else:
            cam.fNumber = 0
        
        
    else:
        cam.name="IR view"
        camM = area.spaces.active.region_3d.view_matrix.inverted()
    #camM = area.spaces.active.region_3d.view_matrix.inverted()
        for region in area.regions:
            if region.type == 'WINDOW':
                viewWidth = region.width
                viewHeight = region.height
                break
        cam.pixels = int(viewWidth/resolutionDivider)
        cam.lines = int(viewHeight/resolutionDivider)
        
        if viewWidth > viewHeight:
            fac = float(viewWidth) / float(viewHeight)
        else:
         fac = 1
        
        offset = area.spaces.active.region_3d.view_camera_offset
        
        
        cam.focalLength = camData.lens / 2        
        cam.shiftLensX =  0
        cam.shiftLensY =  0
        if (area.spaces.active.region_3d.view_perspective == 'CAMERA'):
            zoom = area.spaces.active.region_3d.view_camera_zoom;
            zoom = (1.41421 + zoom/50.0)
            zoom *= zoom
            zoom = 2.0/zoom
            #print("zoom: ",zoom, round(zoom, 5), " offset: ",offset[0],offset[1])        
            cam.focalLength = camData.lens / zoom /2        
            cam.shiftLensX =  32 * offset[0] / zoom
            cam.shiftLensY =  (32 * offset[1] * -1) / fac / zoom
    
        print("cam.focalLength: ", cam.focalLength, camData.lens,area.spaces.active.region_3d.view_camera_zoom, fac, " offset: ",cam.shiftLensX, cam.shiftLensY)
        cam.focusDistance = 0
        cam.filmHeight = 32 / fac      
        cam.fNumber = 0      
    
    

    cam.frame = Transform(\
                camM[0][0], -camM[0][1], -camM[0][2], 
                camM[1][0], -camM[1][1], -camM[1][2], 
                camM[2][0], -camM[2][1], -camM[2][2],
                camM[0][3],  camM[1][3],  camM[2][3])
    irFile = open(os.path.join(exportPath, 'ir.xml'), "w")
    irFile.write('<Root Label=\"Kernel\" Name=\"\" Type=\"Kernel\">\n')
    irFile.write('<Object Identifier=\"./Scenes/%s\" Label=\"Default Scene\" Name=\"%s\" Type=\"Scene\">\n' % ("Blender Scene","Blender Scene"))
    cam.write(irFile)
    display = DisplayOptions()
    display.iso = scene.thea_DispISO
    display.gamma = scene.thea_DispGamma
    display.brightness = scene.thea_DispBrightness
    display.shutter = scene.thea_DispShutter
    display.fNumber = scene.thea_DispFNumber
    display.bloom = scene.thea_DispBloom
    display.bloomWeight = scene.thea_DispBloomWeight / 100
    display.glareRadius = scene.thea_DispGlareRadius / 100
    display.vignetting = scene.thea_DispVignetting / 100
    display.vignettingWeight = scene.thea_DispVignettingWeight / 100    
    display.minZ = scene.thea_DispMinZ
    display.maxZ = scene.thea_DispMaxZ
    display.write(irFile)
    irFile.write("<Parameter Name=\"./Cameras/Active\" Type=\"String\" Value=\"%s\"/></Object>" % cam.name)
    irFile.write("<Parameter Name=\"./Scenes/Active\" Type=\"String\" Value=\"Blender Scene\"/>")
    irFile.write('</Root>\n')
    irFile.close()
    del(cam)
    del(display)

class RENDER_PT_thea_startIR(bpy.types.Operator):
    bl_idname = "thea.start_ir"
    bl_label = "Start IR" 
    START_DELAY = 0.1
    DELAY = START_DELAY
    _handle = None
    _timer = None  
    region = None    
    area = None
    camMatrix = None
    view3dZoom = None
    exportPath = None 
    theaPath = None 
    theaDir = None 
    dataPath = None 
    currentBlendDir = None 
    currentBlendFile = None
    lastPreview = None
    previewAlpha = 1
    img = None
    _updating = False
    resolutionDivider = 1
    IRStartTime = 0
 
    
    @staticmethod
    def handle_add(self, context):        
        RENDER_PT_thea_startIR._handle = bpy.types.SpaceView3D.draw_handler_add(InitGLOverlay, (self, context, self.region, context.area), 'WINDOW', 'POST_PIXEL')
        RENDER_PT_thea_startIR._timer = context.window_manager.event_timer_add(self.DELAY, context.window)
 
    @staticmethod
    def handle_remove(context):
        if RENDER_PT_thea_startIR._handle is not None:
            context.window_manager.event_timer_remove(RENDER_PT_thea_startIR._timer)
            bpy.types.SpaceView3D.draw_handler_remove(RENDER_PT_thea_startIR._handle, 'WINDOW')
        RENDER_PT_thea_startIR._handle = None
        RENDER_PT_thea_startIR._timer = None
    
    @classmethod
    def poll(cls, context):
        area_types = {'VIEW_3D',} 
        return (context.area.type in area_types) and \
               (context.region.type == "WINDOW")
               
    def updateCamera(self, context):
        
        port = context.scene.thea_SDKPort           
        data = sendSocketMsg('localhost', port, b'version')                
                                        
        if data.find('v'):    
            area = context.area
            region = context.region
            scn = context.scene                
            offset = area.spaces.active.region_3d.view_camera_offset

            self.camMatrix = context.area.spaces.active.region_3d.view_matrix.inverted()
            self.view3dZoom = copy.deepcopy(area.spaces.active.region_3d.view_camera_zoom)
            self.view3dOffset[0] = area.spaces.active.region_3d.view_camera_offset[0]
            self.view3dOffset[1] = area.spaces.active.region_3d.view_camera_offset[1]
            
            if context.scene.get('thea_IRResolution'):
                if context.scene.get('thea_IRResolution') == 0:
                    self.resolutionDivider = 1
                if context.scene.get('thea_IRResolution') == 1:
                    self.resolutionDivider = 2
                if context.scene.get('thea_IRResolution') == 2:
                    self.resolutionDivider = 4
            
            thea_globals.frame_px = view3d_camera_border(bpy.context.scene, area)
            if thea_globals.frame_px:
                camM = scn.camera.matrix_world
                x1 = thea_globals.frame_px[3][0]
                y2 = thea_globals.frame_px[3][1]
                x2 = thea_globals.frame_px[0][0]
                y1 = thea_globals.frame_px[1][1]       
                resX = int((x2-x1)/self.resolutionDivider)
                resY = int((y2-y1)/self.resolutionDivider)
                message = 'set "./UI/Viewport/Camera/Active" = "%s"' % (bpy.context.scene.camera.name)
            else:
                camM = area.spaces.active.region_3d.view_matrix.inverted()
                resX = int(context.region.width/self.resolutionDivider)
                resY = int(context.region.height/self.resolutionDivider)
                #message = 'set "./Cameras/Active" = "%s"' % ("3D View")
                message = 'set "./UI/Viewport/Camera/Active" = "%s"' % ("IR view")
            data = sendSocketMsg('localhost', port, message.encode())
            print("message: ",message, data)
            
            message = 'set "./UI/Viewport/Camera/Z-Clipping Near Distance" = %s' % area.spaces.active.clip_start
            data = sendSocketMsg('localhost', port, message.encode())
            print("message: ",message, data)
            
            message1 = 'set "./UI/Viewport/Camera/Frame" = "%s %s %s %s %s %s %s %s %s %s %s %s"' % (camM[0][0], -camM[0][1], -camM[0][2], camM[0][3], 
                                                                                                     camM[1][0], -camM[1][1], -camM[1][2], camM[1][3], 
                                                                                                     camM[2][0], -camM[2][1], -camM[2][2], camM[2][3])
            message2 = 'message "./UI/Viewport/Theme/RW_CMFR"' 
            
            data = sendSocketMsg('localhost', port, message1.encode())
            #print("message: ", message1, data)
            
            data = sendSocketMsg('localhost', port, message2.encode(), waitForResponse=True)
            #print("message: ", message2, data)
            
            if getattr(context.scene, "thea_IRShowTheaWindow", False) == False:
                message3 = 'set "./UI/Viewport/Camera/Resolution" = "%sx%s"' % (resX, resY)
#                message = ('%s\n%s\n%s' % (message1, message3, message2)).encode()
                data = sendSocketMsg('localhost', port, message3.encode(), waitForResponse=True)
                #print("message: ", message3, data)



    def updateObjectTransform(self, context):
        
        port = context.scene.thea_SDKPort           
        data = sendSocketMsg('localhost', port, b'version')                
                                        
        if data.find('v'):    
            area = context.area
            region = context.region
            scn = context.scene                
            object = scn.objects.active
            if len(getattr(object, 'children')) == 0:
                obM = object.matrix_world
                if object.type == "MESH":
                    message = 'set "./Scenes/Active/Models/%s/Frame" = "%s %s %s %s %s %s %s %s %s %s %s %s"' % (object.name, obM[0][0], obM[0][1], obM[0][2], obM[0][3], 
                                                                                                                          obM[1][0], obM[1][1], obM[1][2], obM[1][3], 
                                                                                                                          obM[2][0], obM[2][1], obM[2][2], obM[2][3])
                elif object.type == "LAMP":
                    message = 'set "./Scenes/Active/Lights/%s/Frame" = "%s %s %s %s %s %s %s %s %s %s %s %s"' % (object.name, obM[0][0], obM[0][1], -obM[0][2], obM[0][3], 
                                                                                                                          obM[1][0], obM[1][1], -obM[1][2], obM[1][3], 
                                                                                                                          obM[2][0], obM[2][1], -obM[2][2], obM[2][3])                            
                else:
                    return
                data = sendSocketMsg('localhost', port, message.encode())
                print("message: ",message)
                print("data: ", data)
            else:
                for object in getattr(object, 'children'):
                    obM = object.matrix_world
                    if object.type == "MESH":
                        message = 'set "./Scenes/Active/Models/%s/Frame" = "%s %s %s %s %s %s %s %s %s %s %s %s"' % (object.name, obM[0][0], obM[0][1], obM[0][2], obM[0][3], 
                                                                                                                              obM[1][0], obM[1][1], obM[1][2], obM[1][3], 
                                                                                                                              obM[2][0], obM[2][1], obM[2][2], obM[2][3])
                        data = sendSocketMsg('localhost', port, message.encode())
                    elif object.type == "LAMP":
                        message = 'set "./Scenes/Active/Lights/%s/Frame" = "%s %s %s %s %s %s %s %s %s %s %s %s"' % (object.name, obM[0][0], obM[0][1], -obM[0][2], obM[0][3], 
                                                                                                                              obM[1][0], obM[1][1], -obM[1][2], obM[1][3], 
                                                                                                                              obM[2][0], obM[2][1], -obM[2][2], obM[2][3]) 
                        data = sendSocketMsg('localhost', port, message.encode())

            
            message = 'message "./UI/Viewport/Tweak Position"'             
            data = sendSocketMsg('localhost', port, message.encode())
            #print("message: ",message)
            #print("data: ", data)
            #return
            if data.find('Ok'):
                #switch to camera frame
                message = b'message "./UI/Viewport/Theme/RW_CMFR"'                                                
                data = sendSocketMsg('localhost', port, message)
                #print("message: ",message)
                #print("data: ", data)
                #set resolution
                if getattr(context.scene, "thea_IRShowTheaWindow", False) == False:
                    message = 'set "./UI/Viewport/Camera/Resolution" = "%sx%s"' % (int(context.region.width/self.resolutionDivider),int(context.region.height/self.resolutionDivider))
#                         print("message: ",message)
                    data = sendSocketMsg('localhost', port, message.encode())
                #print("message: ",message)
                #print("data: ", data)
                bpy.context.scene.thea_ir_update = True    
                draw_IR_preview(self, context, self.region, self.previewAlpha, self.img, area) 

                
    def updateEnvironment(self, context):
        
        port = context.scene.thea_SDKPort
        scn = context.scene
        environmentOptions=EnvironmentOptions()
        environmentOptions.globalFrame=Transform()
        if (scn.thea_IBLEnable): # if IBL is on
            print("IBL")
            iblMap = EnvironmentOptions.IBLMap()
            iblMap.bitmapTexture = BitmapTexture(bpy.path.abspath(scn.thea_IBLFilePath))
            iblMap.iblType = scn.thea_IBLTypeMenu
            iblMap.wrapping = scn.thea_IBLWrappingMenu            
            iblMap.enabledParameter = True
            iblMap.intensity = scn.thea_IBLIntensity
            iblMap.rotation = scn.thea_IBLRotation
            environmentOptions.illuminationMap=iblMap
            environmentOptions.illumination="IBL"
        else:
            exporter.getEnvironmentOptions().illumination="None"
                
        if scn.thea_EnvPSEnable:
            environmentOptions.illumination="PhysicalSky"

        environmentOptions.turbidity = scn.thea_EnvPSTurb
        environmentOptions.waterVapor = scn.thea_EnvPSWatVap
        environmentOptions.turbidityCoefficient = scn.thea_EnvPSTurbCo
        environmentOptions.wavelengthExponent = scn.thea_EnvPSWaveExp
        environmentOptions.latitude = scn.thea_EnvLat
        environmentOptions.longitude = scn.thea_EnvLong
        environmentOptions.timezone = ("GT+"+scn.thea_EnvTZ if int(scn.thea_EnvTZ)>0 else "GT"+scn.thea_EnvTZ)
        environmentOptions.localtime = scn.thea_EnvTime
        environmentOptions.date = scn.thea_EnvDate
        
        import tempfile
        tempDir = tempfile.gettempdir()
        envFilename = os.path.join(tempDir,'env.xml')
        envFile = open(envFilename,"w")
        
        environmentOptions.write(envFile)
        
                
        message = b'message "./UI/Viewport/Theme/RW_PROG"'        
        data = sendSocketMsg('localhost', port, message)
        print("message: ", message)
        print("data: ", data)
        if data.find('Ok'):
            message = ('message "Merge \'%s\' 0 0 0 1 0 0"' % os.path.join(self.exportPath, 'env.xml')).encode() 
            data = sendSocketMsg('localhost', port, message)                 
            print("message: ", message)
            print("data: ", data)                 
            if data.find('Ok'):
                print("Scene merged")
                message = b'message "./UI/Viewport/Theme/RW_EARF"'
                data = sendSocketMsg('localhost', port, message)
                print("message: ", message)
                print("data: ", data)
                if data.find('Ok'):
                    print("\n\n***Interactive render started!****\n\n")
        
        

    def updateMaterial(self, context):
        
        print("******************* update material ***************")
        port = context.scene.thea_SDKPort             
        area = context.area
        region = context.region
        scn = context.scene                
        offset = area.spaces.active.region_3d.view_camera_offset

        material = scn.objects.active.active_material    
        import tempfile
        tempDir = tempfile.gettempdir()
            
        matFilename = os.path.join(tempDir,'material_preview.xml')
        matFile = open(matFilename,"w")
        
        #print("material: , ", material)
        
        ma = ThMaterial()
        ma.setName(material.name)
        ma.blenderMat = material    
        if material:
            exporter = XMLExporter()
            thMat = exporter.generateMaterialNew(ma)            
            ma.write(matFile, preview=True)

        #message = b'message "./UI/Viewport/Theme/RW_PROG"'
        #print("message: ",message)
        #data = sendSocketMsg('localhost', port, message)
        #print("data: ", data)
        message = "message \"LoadObject './Scenes/Active/Proxies/Appearance/%s' '%s'\"" % (material.name, matFilename)
        #message = b'message \"LoadObject \"./Scenes/Active/Proxies/Appearance/somematerial\" \"/tmp/material_preview.xml\"\"'
        #message = ('set "./Scenes/Active/Proxies/Appearance/%s/Diffuse/Constant Texture/" = "0.5 0.5 0.0"' % (material.name)).encode()        
        #message = ('message "Merge %s 0 0 0 0 0 4"' % matFilename).encode()
        print("message: ",message)        
        data = sendSocketMsg('localhost', port, message)
        print("data: ", data)
        message = b'message "./UI/Viewport/Tweak Material"'
        print("message: ",message)
        data = sendSocketMsg('localhost', port, message)
        #print("data: ", data)        
        #message = b'message "./UI/Viewport/Theme/RW_EARF"'
        #print("message: ",message)
        #data = sendSocketMsg('localhost', port, message)
        #print("data: ", data)
        return

        

               
    
    def modal(self, context, event):
   
        port = context.scene.thea_SDKPort
        if context.area:
            context.area.tag_redraw()  
        

        if event.type == 'TIMER':   
            area = context.area
            region = context.region
            #print("materialUpdatesNumber: ", thea_globals.materialUpdatesNumber)
            if thea_globals.materialUpdatesNumber > 2:
                thea_globals.materialUpdatesNumber -= 1
                
            if (len(thea_globals.objectsUpdated) > 0):
                print("Object updated *****************************")
                if getattr(context.scene, "thea_IRFullUpdate", False):
                    refreshIR()
                else:
                    self.updateObjectTransform(context)
                thea_globals.objectsUpdated = []
            
#             if thea_globals.worldUpdated:
#                 self.updateEnvironment(context)
#                 thea_globals.worldUpdated = False
            if thea_globals.materialUpdated:# and thea_globals.materialUpdatesNumber<=2:
                #if (os.times()[4]-self.IRStartTime)>5:
                #if thea_globals.materialUpdated:                    
                #self.updateMaterial(context)
                thea_updateIR(context)
                print("\n\n\n\n************** material updated *********************\n\n\n\n: ", context.scene.objects.active.active_material)
                thea_globals.materialUpdated = False
                time.sleep(0.2)
                    #self.IRStartTime = os.times()[4]
#                 else:
#                     thea_globals.materialUpdated = False
            try:
                if (self.camMatrix != area.spaces.active.region_3d.view_matrix.inverted()) or (self.view3dZoom != area.spaces.active.region_3d.view_camera_zoom) or (self.view3dOffset[0] != area.spaces.active.region_3d.view_camera_offset[0]) and not self._updating:
                    camFrameChanged = True
                else:
                    camFrameChanged = False
            except:
                camFrameChanged = False
            #if (self.camMatrix != area.spaces.active.region_3d.view_matrix.inverted()) or (self.view3dZoom != area.spaces.active.region_3d.view_camera_zoom) or (self.view3dOffset[0] != area.spaces.active.region_3d.view_camera_offset[0]) and not self._updating:
            if camFrameChanged:
                self.camMatrix = context.area.spaces.active.region_3d.view_matrix.inverted()
                self.view3dZoom = copy.deepcopy(area.spaces.active.region_3d.view_camera_zoom)
                #self.view3dOffset = copy.deepcopy(area.spaces.active.region_3d.view_camera_offset)
                self.view3dOffset[0] = area.spaces.active.region_3d.view_camera_offset[0]
                self.view3dOffset[1] = area.spaces.active.region_3d.view_camera_offset[1]
                self._updating = True
                self.updateCamera(context)
                if(getattr(context.scene, "thea_IRBlendAlpha", False)):
                    thea_globals.preview3DAlpha = 0.0
                self._updating = False
            
            if thea_globals.engineUpdated:
                print("thea_globals.engineUpdated: ", thea_globals.engineUpdated)
                message = b'message "./UI/Viewport/Theme/RW_PROG"'        
                data = sendSocketMsg('localhost', port, message)
                print("message: ", message)
                print("data: ", data)
                if data.find('Ok'):
                    message = ('set "./Scenes/Active/Render/Interactive/Engine Core" = "%s"' % getattr(context.scene, 'thea_IRRenderEngineMenu')).encode() 
                    data = sendSocketMsg('localhost', port, message)                 
                    print("message: ", message)
                    print("data: ", data)                 
                    if data.find('Ok'):
                        print("Scene merged")
                        message = b'message "./UI/Viewport/Theme/RW_EARF"'
                        data = sendSocketMsg('localhost', port, message)
                        print("message: ", message)
                        print("data: ", data)
                        if data.find('Ok'):
                            print("\n\n***Interactive render started!****\n\n")
                thea_globals.engineUpdated = False
            
            if thea_globals.displayUpdated:
                print("thea_globals.displayUpdated: ", thea_globals.displayUpdated)
                from TheaForBlender.thea_render_main import updateDisplaySettings 
                updateDisplaySettings(port)

                thea_globals.displayUpdated = False             
            
#             if getattr(context.scene, "thea_DrawPreviewto3dView", False) or getattr(context.scene, "thea_IRShowTheaWindow", False)==False:
            if getattr(context.scene, "thea_SavePreviewtoImage", False) or getattr(context.scene, "thea_IRShowTheaWindow", False)==False:                
                message = 'message "./UI/Viewport/SaveImage %s"' % os.path.join(self.exportPath, thea_globals.IrPreviewFileName) 
                data = sendSocketMsg('localhost', port, message.encode())
            for area in bpy.context.screen.areas:
                if area.type == "IMAGE_EDITOR":
                    area.tag_redraw()
            
            thea_globals.preview3DAlpha += 0.25
            self.lastPreview = os.times()[4]   
            
            try:
                img = bpy.data.images[thea_globals.IrPreviewFileName]            
            except:
                try:
                    img = bpy.data.images.load(os.path.join(exportPath, thea_globals.IrPreviewFileName))
                except:
                    img = None
                    return {'PASS_THROUGH'}
                
            if os.path.exists(os.path.join(self.exportPath, thea_globals.IrPreviewFileName)):
                fileSize = os.path.getsize(os.path.join(self.exportPath, thea_globals.IrPreviewFileName))
            else:
                fileSize = 0
            minSize = ((img.size[0]*img.size[1])/50)+thea_globals.minIRpreviewSize
            #print("fileSize, minSize: ", fileSize, minSize) 
            if (fileSize>((img.size[0]*img.size[1])/50)+thea_globals.minIRpreviewSize): #if file is very small, it's probably empty
                img.reload()
            
#             try:
#                 self.img = bpy.data.images[thea_globals.IrPreviewFileName]
#                 #print("ir.jpg size: ", os.path.getsize(os.path.join(self.exportPath, thea_globals.IrPreviewFileName)))
#                 self.img.reload()
#             except:
#                 try:
#                     self.img = bpy.data.images.load(os.path.join(self.exportPath, thea_globals.IrPreviewFileName))
#                 except:
#                     pass

            return {'PASS_THROUGH'}

        data = sendSocketMsg('localhost', port, b'version')
    
        if data.find('ERROR') >= 0:              
            context.scene.thea_ir_running = False
            thea_globals.IrIsRunning = False      
            self.report({'ERROR'}, "Connection with RemoteDarkroom has been lost, exiting IR!")
            return {'FINISHED'}
        
        if not context.scene.thea_ir_running:
            port = context.scene.thea_SDKPort
            
            data = sendSocketMsg('localhost', port, b'version')
    
            if data.find('v'):        
                message = b'message "./UI/Viewport/Theme/RW_PROG"' 
                data = sendSocketMsg('localhost', port, message)
                message = b'message "exit"' 
                data = sendSocketMsg('localhost', port, message)                
            #context.window_manager.event_timer_remove(self._timer)
            return {'CANCELLED'}
        return {'PASS_THROUGH'}
    
    
    def invoke(self, context, event):
        
        
        
        self.view3dOffset = [0,0]
        thea_globals.preview3DAlpha = 0.0
        thea_globals.materialUpdated = False
        IrIsRunning = False
        thea_globals.objectsUpdated = []        
        
        alpha = bpy.context.scene.get('thea_IRAlpha')
        if (bpy.context.scene.get('thea_IRAlpha') == None):
            bpy.context.scene['thea_IRAlpha'] = 1.0
        (self.exportPath, self.theaPath, self.theaDir, self.dataPath, self.currentBlendDir, self.currentBlendFile) = setPaths(context.scene)
        
        try:
            if bpy.context.active_object.mode != 'OBJECT':
                self.report({'ERROR'}, "Please change to object mode before rendering!")  
                return {'FINISHED'}
        except:
            pass
        
        if len(self.currentBlendFile)<2:
            self.report({'ERROR'}, "Please save the scene before exporting!")  
            return {'FINISHED'}
        from TheaForBlender.thea_render_main import checkTheaMaterials
        if(checkTheaMaterials()==False):
            self.report({'ERROR'}, "Please set materials and lights to get proper render")  
            #return {'FINISHED'}
        if not os.path.isdir(self.exportPath):
            self.report({'ERROR'}, "Please set proper output path before exporting!")  
            return {'FINISHED'}
        
        if context.area.type == 'VIEW_3D':
                                    
#             print("############ area.spaces.active.local_view: ",context.area.spaces.active.local_view)            
            if context.scene.thea_ir_running == False:
                try:
                    if context.scene.get('thea_SDKPort'):
                        port = context.scene.get('thea_SDKPort')                
                    else:
                        port = 30000
                    #if context.scene.get('thea_RefreshDelay'):
                    #    self.DELAY = context.scene.get('thea_RefreshDelay')                 
                except:
                    port = 30000
                #self.DELAY = self.START_DELAY
                self.DELAY = getattr(context.scene, 'thea_RefreshDelay')
                socketServerIsRunning = False
                data = sendSocketMsg('localhost', port, b'version')
                print("data: ",data)
                if data.find('v')>0:
                    socketServerIsRunning = True                    
                else:
                    p = startTheaRemoteDarkroom(port, show=getattr(context.scene, "thea_IRShowTheaWindow", False), idletimeout=getattr(context.scene, 'thea_IRIdleTimeout', 120))
                    #print("process: ",p)
                    #time.sleep(bpy.context.scene.thea_StartTheaDelay)
                    message = b'version'
                    data = sendSocketMsg('localhost', port, message)
                    #print("data: ",data)
                    if data.find('v')>0:
                        socketServerIsRunning = True
                
                print("socketServerIsRunning: ",socketServerIsRunning)
                if socketServerIsRunning:        
                    frame = context.scene.frame_current
                    frameS= "0000" + str(frame)
                    xmlFilename = self.currentBlendFile.replace('.blend', '.xml')
#                     os.chdir(os.path.dirname(self.theaPath))
                    exporter=initExporter()
                    context.scene.thea_IRMessage = "Exporting data..."
                    
                    #os.chdir(context.scene.render.filepath)
                    os.chdir(self.exportPath)
                    self.region = context.region
                    context.scene.thea_ir_running = True
                    area = context.area
                    
                    currExportAnimation = getattr(bpy.context.scene, 'thea_ExportAnimation')
                    if getattr(bpy.context.scene, 'thea_IRExportAnimation', False):
                        bpy.context.scene.thea_ExportAnimation = True
                    else:
                        bpy.context.scene.thea_ExportAnimation = False
                    args = exportFrame(context.scene,context.scene.frame_current,exporter=exporter, area=area)
                    bpy.context.scene.thea_ExportAnimation = currExportAnimation
                    context.scene.thea_IRMessage = ""
                    self.camMatrix = context.area.spaces.active.region_3d.view_matrix.inverted()
                    self.view3dZoom = area.spaces.active.region_3d.view_camera_zoom
                    self.view3dOffset[0] = area.spaces.active.region_3d.view_camera_offset[0]
                    self.view3dOffset[1] = area.spaces.active.region_3d.view_camera_offset[1]
                    DrawStringToViewport("Starting IR", 100, 100, 50, (1.0, 1.0, 1.0, 1.0))                    
                    thea_globals.IrIsRunning = True       
                    print("getattr(scn, 'thea_SceneMerReverseOrder'): ", getattr(bpy.context.scene, 'thea_SceneMerReverseOrder'))
                    print("getattr(scn, 'thea_mergeFilePath'): ", getattr(bpy.context.scene, 'thea_mergeFilePath'))
                                    
                      #load another scene and then merge exported one
                    if getattr(bpy.context.scene, "thea_showMerge"):
                        if os.path.exists(getattr(bpy.context.scene, 'thea_mergeFilePath')):
                            mergeString = (" %s %s %s %s %s %s" % (bpy.context.scene.thea_SceneMerModels, bpy.context.scene.thea_SceneMerLights, bpy.context.scene.thea_SceneMerCameras, bpy.context.scene.thea_SceneMerRender, bpy.context.scene.thea_SceneMerEnv, bpy.context.scene.thea_SceneMerMaterials))
                            if getattr(bpy.context.scene, 'thea_SceneMerReverseOrder'):                            
#                                 message = ('message "Load %s"' % os.path.join(self.exportPath, os.path.basename(xmlFilename))).encode()
                                message = ('message "Load %s"' % getattr(bpy.context.scene, 'thea_mergeFilePath')).encode()
                                print("message: ", message)
                                data = sendSocketMsg('localhost', port, message)
                                print("data: ", data)
#                                 message = ('message "Merge %s %s' % (getattr(bpy.context.scene, 'thea_mergeFilePath'), mergeString)).encode()
                                message = ('message "Merge %s %s' % (os.path.join(self.exportPath, os.path.basename(xmlFilename)), mergeString)).encode()
                                print("message: ", message)
                                data = sendSocketMsg('localhost', port, message)
                                print("data: ", data)
                            else:
#                                 message = ('message "Merge %s %s' % (getattr(bpy.context.scene, 'thea_mergeFilePath'), mergeString)).encode()
                                message = ('message "Load %s"' % os.path.join(self.exportPath, os.path.basename(xmlFilename))).encode()
                                print("message: ", message)
                                data = sendSocketMsg('localhost', port, message)
                                print("data: ", data)
#                                 message = ('message "Load %s"' % os.path.join(self.exportPath, os.path.basename(xmlFilename))).encode()
                                message = ('message "Merge %s %s' % (getattr(bpy.context.scene, 'thea_mergeFilePath'), mergeString)).encode()
                                print("message: ", message)
                                data = sendSocketMsg('localhost', port, message)
                                print("data: ", data)
                    else:
                        message = ('message "Load %s"' % os.path.join(self.exportPath, os.path.basename(xmlFilename))).encode()
                        print("message: ", message)
                        data = sendSocketMsg('localhost', port, message)
                        print("data: ", data)
                              
#                     message = ('message "Load %s"' % os.path.join(self.exportPath, os.path.basename(xmlFilename))).encode() 
#                     print("message: ", message)
#                     data = sendSocketMsg('localhost', port, message)
#                     print("data: ", data)
                    if data.find('Ok'):
                        print("Scene loaded")
                        self.updateCamera(context)
                        
                         #print("context.scene.get('thea_IRResolution'): ", context.scene.get('thea_IRResolution'))
                        if context.scene.get('thea_IRResolution'):
                            if context.scene.get('thea_IRResolution') == 0:
                                self.resolutionDivider = 1
                            if context.scene.get('thea_IRResolution') == 1:
                                self.resolutionDivider = 2
                            if context.scene.get('thea_IRResolution') == 2:
                                self.resolutionDivider = 4                    
                                
                        print("self.resolutionDivider: ", self.resolutionDivider)
                        #switch to camera frame
                        message = b'message "./UI/Viewport/Theme/RW_CMFR"'                                                
                        data = sendSocketMsg('localhost', port, message)
                        print("message: ",message)
                        #print("out: ", data)
                        #set resolution
                        thea_globals.frame_px = view3d_camera_border(bpy.context.scene, area)
                        if thea_globals.frame_px:
                            camM = bpy.context.scene.camera.matrix_world
                            x1 = thea_globals.frame_px[3][0]
                            y2 = thea_globals.frame_px[3][1]
                            x2 = thea_globals.frame_px[0][0]
                            y1 = thea_globals.frame_px[1][1]       
                            resX = int((x2-x1)/self.resolutionDivider)
                            resY = int((y2-y1)/self.resolutionDivider)
                        else:
                            camM = area.spaces.active.region_3d.view_matrix.inverted()
                            resX = int(context.region.width/self.resolutionDivider)
                            resY = int(context.region.height/self.resolutionDivider)
                        #message = 'set "./UI/Viewport/Camera/Resolution" = "%sx%s"' % (int(context.region.width/self.resolutionDivider),int(context.region.height/self.resolutionDivider))
                        if getattr(context.scene, "thea_IRShowTheaWindow", False) == False:
                            message = 'set "./UI/Viewport/Camera/Resolution" = "%sx%s"' % (resX, resY)
                            print("message res: ",message, message.encode())
                            data = sendSocketMsg('localhost', port, message.encode())                        
                            print("out: ", data)
                        #start auto refresh IR
                        message = b'message "./UI/Viewport/Theme/RW_EARF"'
                        print("message: ",message)
                        data = sendSocketMsg('localhost', port, message)
                        if data.find('Ok'):
                            print("\n\n***Interactive render started!****\n\n")                                    
                            time.sleep(self.DELAY)
                            self.IRStartTime = os.times()[4]
                            print("self.IRStartTime: ", self.IRStartTime)
#                             if getattr(context.scene, "thea_DrawPreviewto3dView", False) or getattr(context.scene, "thea_IRShowTheaWindow", False)==False:
                            if getattr(context.scene, "thea_SavePreviewtoImage", False) or getattr(context.scene, "thea_IRShowTheaWindow", False)==False:
                                message = 'message "./UI/Viewport/SaveImage %s"' % os.path.join(self.exportPath, thea_globals.IrPreviewFileName) 
                                data = sendSocketMsg('localhost', port, message.encode())
                                print("data: ", data)       
                                try:
                                    img = bpy.data.images[thea_globals.IrPreviewFileName]
                                    img.filepath = os.path.join(self.exportPath, thea_globals.IrPreviewFileName)
                                    img.reload()
                                except:
                                    try:
                                        img = bpy.data.images.load(os.path.join(self.exportPath, thea_globals.IrPreviewFileName))
                                    except:
                                        pass                    
                            self.cursor_on_handle = 'None'
                            context.window_manager.modal_handler_add(self)                            
                            self.lastPreview = os.times()[4]
                            self.previewAlpha = 0
                            RENDER_PT_thea_startIR.handle_add(self, context)
                            bpy.context.scene.thea_ir_update = False
                            bpy.context.scene.thea_lastIRUpdate = os.times()[4]                                                        
                            bpy.app.handlers.scene_update_post.append(thea_collectUpdatedObjects)
#                             bpy.app.handlers.scene_update_post.append(thea_sceneUpdated)
                            
                            message = b'version'
                            data = sendSocketMsg('localhost', port, message)
                            #print("data: ",data)
                            if data.find('v1.3')>0:
                                exportIRCamera(context.scene, area, self.exportPath)
                            
                                message = ('message "Merge %s 0 0 1 0 0"' % os.path.join(self.exportPath, 'ir.xml')).encode()                             
                                #message = ('message "Merge %s 0 0 1 1 0"' % os.path.join(self.exportPath, 'ir.xml')).encode()
                                data = sendSocketMsg('localhost', port, message)                            
                                print("message: ",message)
                                if data.find('Ok'):
                                    message = 'set "./Cameras/Active" = "%s"' % ("Camera")
                                    #switch to camera frame
                                    message = b'message "./UI/Viewport/Theme/RW_CMFR"'                                                
                                    data = sendSocketMsg('localhost', port, message)
                                    print("message: ",message)
            #                         print("out: ", data)
                                    #set resolution
                                    if thea_globals.frame_px:
                                        camM = bpy.context.scene.camera.matrix_world
                                        x1 = thea_globals.frame_px[3][0]
                                        y2 = thea_globals.frame_px[3][1]
                                        x2 = thea_globals.frame_px[0][0]
                                        y1 = thea_globals.frame_px[1][1]       
                                        resX = int((x2-x1)/self.resolutionDivider)
                                        resY = int((y2-y1)/self.resolutionDivider)
                                    else:
                                        camM = area.spaces.active.region_3d.view_matrix.inverted()
                                        resX = int(context.region.width/self.resolutionDivider)
                                        resY = int(context.region.height/self.resolutionDivider)
                                    #message = 'set "./UI/Viewport/Camera/Resolution" = "%sx%s"' % (int(context.region.width/self.resolutionDivider),int(context.region.height/self.resolutionDivider))
                                    if getattr(context.scene, "thea_IRShowTheaWindow", False) == False:
                                        message = 'set "./UI/Viewport/Camera/Resolution" = "%sx%s"' % (resX, resY)
                                        print("message: ",message)
                                        data = sendSocketMsg('localhost', port, message.encode())
                                        bpy.context.scene.thea_ir_update = True     
                                        draw_IR_preview(self, context, self.region, self.previewAlpha, self.img, area) 
                                        time.sleep(0.2) #small delay to give thea time to render anything after merge
                                        thea_globals.IrIsRunning = True             
                                        print("data: ", data)
                            
                            return {'RUNNING_MODAL'}
                elif data == b'ERROR':
                    self.report({'WARNING'}, "Thea socket server is not running, can't run operator")
                    thea_globals.IrIsRunning = False
                    return {'CANCELLED'}
            else:
                # operator is called again, stop displaying
                context.scene.thea_ir_running = False
                thea_globals.IrIsRunning = False
                RENDER_PT_thea_startIR.handle_remove(context)
                return {'CANCELLED'}
        
        else:
            self.report({'WARNING'}, "View3D not found, can't run operator")
            return {'CANCELLED'}



def DrawStringToViewport(text, x, y, size, color):
    ''' my_string : the text we want to print
        pos_x, pos_y : coordinates in integer values
        size : font height.
        colour_type : used for definining the colour'''
    my_dpi, font_id = 72, 0 # dirty fast assignment
    bgl.glColor4f(*color)
    blf.position(font_id, x, y, 0)
    blf.size(font_id, size, my_dpi)
    blf.draw(font_id, text)

def view3d_find(area):
    # returns first 3d view, normally we get from context
    #for area in bpy.context.window.screen.areas:
    if area.type == 'VIEW_3D':
        v3d = area.spaces[0]
        rv3d = v3d.region_3d
        for region in area.regions:
            if region.type == 'WINDOW':
                viewM = area.spaces.active.region_3d.view_matrix
                camM = bpy.context.scene.camera.matrix_world.inverted()
                #print("camM: ", camM)
                #print("viewM: ", viewM)
                sum = 0
                for v in (viewM-camM):
                    for i in v:
                        sum += abs(i)
                #print("sum: ", sum)
                if sum<1e-5:
                    return region, rv3d
    return None, None

def view3d_camera_border(scene, area):
    obj = scene.camera
    cam = obj.data

    frame = cam.view_frame(scene)

    # move into object space
    frame = [obj.matrix_world * v for v in frame]

    # move into pixelspace
    from bpy_extras.view3d_utils import location_3d_to_region_2d
    region, rv3d = view3d_find(area)
    if region:
        frame_px = [location_3d_to_region_2d(region, rv3d, v) for v in frame]
        return frame_px
    else:
        return False

def draw_IR_preview(self, context, region, previewAlpha, img, area):
    #global preview3DAlpha
    
    
        
    if region.type == 'WINDOW':
            
        viewWidth = region.width
        viewHeight = region.height
            
        img = 0                
        try:
            img = bpy.data.images[thea_globals.IrPreviewFileName]            
        except:
            try:
                img = bpy.data.images.load(os.path.join(exportPath, thea_globals.IrPreviewFileName))
            except:
                img = None
                return
        if os.path.exists(os.path.join(self.exportPath, thea_globals.IrPreviewFileName)):
            fileSize = os.path.getsize(os.path.join(self.exportPath, thea_globals.IrPreviewFileName))
        else:
            fileSize = 0
        minSize = ((img.size[0]*img.size[1])/50)+thea_globals.minIRpreviewSize
        #print("fileSize, minSize: ", fileSize, minSize) 
        if (fileSize>((img.size[0]*img.size[1])/50)+thea_globals.minIRpreviewSize): #if file is very small, it's probably empty
            img.reload()
            thea_globals.frame_px = view3d_camera_border(bpy.context.scene, area)

            if img.size[0]>0:      
                if thea_globals.frame_px:
                    x1 = thea_globals.frame_px[3][0]
                    y2 = thea_globals.frame_px[3][1]
                    x2 = thea_globals.frame_px[0][0]
                    y1 = thea_globals.frame_px[1][1]                
                else: 
                    x1 = 0
                    y1 = 0
                    x2 = viewWidth
                    y2 = viewHeight
                color=[1,1,1,1]
                img.gl_load(bgl.GL_NEAREST, bgl.GL_NEAREST)
                if hasattr(img.bindcode, "__len__"): #fix for change in 2.77 
                    bindcode = img.bindcode[0]
                else:
                    bindcode = img.bindcode
                bgl.glBindTexture(bgl.GL_TEXTURE_2D, bindcode)
                bgl.glEnable(bgl.GL_TEXTURE_2D)
                bgl.glEnable(bgl.GL_BLEND)
                bgl.glColor4f(1,1,1,max(min(thea_globals.preview3DAlpha, 1), 0))
                #bgl.glColor4f(1,1,1,0.5)
                bgl.glBegin(bgl.GL_QUADS)
                bgl.glTexCoord2f(0,0)
                bgl.glVertex2f(x1,y1)
                bgl.glTexCoord2f(0,1)
                bgl.glVertex2f(x1,y2)
                bgl.glTexCoord2f(1,1)
                bgl.glVertex2f(x2,y2)
                bgl.glTexCoord2f(1,0)
                bgl.glVertex2f(x2,y1)        
                bgl.glEnd()
        
        
def InitGLOverlay(self, context, region, area):

    if context.scene.get('thea_DrawPreviewto3dView') and context.scene.thea_ir_running:
        #draw_IR_preview(self, context, self.region,1,False)
        draw_IR_preview(self, context, self.region,thea_globals.preview3DAlpha,False,area)
    bgl.glEnable(bgl.GL_BLEND)
    return


def refreshIR():
    
    region = None
    resolutionDivider = 1
    
    (exportPath, theaPath, theaDir, dataPath, currentBlendDir, currentBlendFile) = setPaths(bpy.context.scene)
    
    #global preview3DAlpha
    thea_globals.preview3DAlpha = 0.0
    
    if bpy.context.area.type == 'VIEW_3D':
        if bpy.context.scene.thea_ir_running == True:
            try:
                if bpy.context.scene.get('thea_SDKPort'):
                    port = context.scene.get('thea_SDKPort')                
                else:
                    port = 30000
                #if context.scene.get('thea_RefreshDelay'):
                #    self.DELAY = context.scene.get('thea_RefreshDelay')                 
            except:
                port = 30000
                   
            data = sendSocketMsg('localhost', port, b'version')
    
            if data.find('v'):       
                
                message = b'message "./UI/Viewport/Theme/RW_PROG"'
                data = sendSocketMsg('localhost', port, message)
                message = b'message "Reset"'         
                data = sendSocketMsg('localhost', port, message)
                if data.find('Ok'):
                    print("\n\n***Interactive render stopped!****\n\n")
                    frame = bpy.context.scene.frame_current
                    frameS= "0000" + str(frame)
                    xmlFilename = currentBlendFile.replace('.blend', '.xml')
                    os.chdir(os.path.dirname(theaPath))
                    exporter=initExporter() 
                    #os.chdir(context.scene.render.filepath)
                    os.chdir(exportPath)
                    region = bpy.context.region
                    area = bpy.context.area
    #                         bpy.context.scene['thea_Reuse'] = True
                    currExportAnimation = getattr(bpy.context.scene, 'thea_ExportAnimation')
                    if getattr(bpy.context.scene, 'thea_IRExportAnimation', False):
                        bpy.context.scene.thea_ExportAnimation = True
                    else:
                        bpy.context.scene.thea_ExportAnimation = False
                    args = exportFrame(bpy.context.scene,bpy.context.scene.frame_current,exporter=exporter, area=area)
                    bpy.context.scene.thea_ExportAnimation = currExportAnimation 
                    
    #                         bpy.context.scene['thea_Reuse'] = False
    #                         message = ('message "Merge %s 4 1 2 1 1 4"' % os.path.join(exportPath, os.path.basename(xmlFilename))).encode()
                    message = ('message "Load %s"' % os.path.join(exportPath, os.path.basename(xmlFilename))).encode()
    #                         message = ('message "Load %s"' % os.path.join(exportPath, os.path.basename(xmlFilename))).encode() 
                    print("message: ", message)
                    data = sendSocketMsg('localhost', port, message)
                    print("data: ", data)
                    if data.find('Ok'):
                        print("Scene merged")
                        if bpy.context.scene.get('thea_IRResolution'):
                            if bpy.context.scene.get('thea_IRResolution') == 0:
                                resolutionDivider = 1
                            if bpy.context.scene.get('thea_IRResolution') == 1:
                                resolutionDivider = 2
                            if bpy.context.scene.get('thea_IRResolution') == 2:
                                resolutionDivider = 4      
             
                        if thea_globals.frame_px:
                            camM = bpy.context.scene.camera.matrix_world
                            x1 = thea_globals.frame_px[3][0]
                            y2 = thea_globals.frame_px[3][1]
                            x2 = thea_globals.frame_px[0][0]
                            y1 = thea_globals.frame_px[1][1]       
                            resX = int((x2-x1)/resolutionDivider)
                            resY = int((y2-y1)/resolutionDivider)
                            message = 'set "./UI/Viewport/Camera/Active" = "%s"' % (bpy.context.scene.camera.name)
                        else:
                            camM = area.spaces.active.region_3d.view_matrix.inverted()
                            resX = int(bpy.context.region.width/resolutionDivider)
                            resY = int(bpy.context.region.height/resolutionDivider)
                            #message = 'set "./Cameras/Active" = "%s"' % ("3D View")
                            message = 'set "./UI/Viewport/Camera/Active" = "%s"' % ("IR view")
                        data = sendSocketMsg('localhost', port, message.encode())
                        print("message: ",message, data)
                                                
                        #switch to camera frame                            
                        message = b'message "./UI/Viewport/Theme/RW_CMFR"'                                                
                        data = sendSocketMsg('localhost', port, message)
                        print("message: %s, data: %s|" % (message, data))

                        
                        message1 = 'set "./UI/Viewport/Camera/Frame" = "%s %s %s %s %s %s %s %s %s %s %s %s"' % (camM[0][0], -camM[0][1], -camM[0][2], camM[0][3], 
                                                                                                                 camM[1][0], -camM[1][1], -camM[1][2], camM[1][3], 
                                                                                                                 camM[2][0], -camM[2][1], -camM[2][2], camM[2][3])
                        data = sendSocketMsg('localhost', port, message1.encode())
                        #set resolution
                        if getattr(bpy.context.scene, "thea_IRShowTheaWindow", False) == False:
                            message = 'set "./UI/Viewport/Camera/Resolution" = "%sx%s"' % (int(bpy.context.region.width/resolutionDivider),int(bpy.context.region.height/resolutionDivider))
                            print("message: ",message)
                            data = sendSocketMsg('localhost', port, message.encode())                        
                            print("out: ", data)
                        message = b'message "./UI/Viewport/Theme/RW_EARF"'
                        data = sendSocketMsg('localhost', port, message)
                        if data.find('Ok'):
                            print("\n\n***Interactive render started!****\n\n")                                    
                            return {'FINISHED'}
                    elif data == b'ERROR':
#                         self.report({'WARNING'}, "Thea socket server is not running, can't run operator")
                        return {'CANCELLED'}
                else:
#                     self.report({'WARNING'}, "IR is not currently running")
                    return {'CANCELLED'}
            else:
#                 self.report({'WARNING'}, "View3D not found, can't run operator")
                return {'CANCELLED'}

    
class RENDER_PT_thea_refreshIR(bpy.types.Operator):
    bl_idname = "thea.refresh_ir"
    bl_label = "Refresh IR preview"    
    #DELAY = 2
    _handle = None
    _timer = None  
    

    def invoke(self, context, event):
        return refreshIR()
#         (exportPath, theaPath, theaDir, dataPath, currentBlendDir, currentBlendFile) = setPaths(context.scene)
#         
#         #global preview3DAlpha
#         thea_globals.preview3DAlpha = 0.0
#         
#         if context.area.type == 'VIEW_3D':
#             if context.scene.thea_ir_running == True:
#                 try:
#                     if context.scene.get('thea_SDKPort'):
#                         port = context.scene.get('thea_SDKPort')                
#                     else:
#                         port = 30000
#                     #if context.scene.get('thea_RefreshDelay'):
#                     #    self.DELAY = context.scene.get('thea_RefreshDelay')                 
#                 except:
#                     port = 30000
#                        
#                 data = sendSocketMsg('localhost', port, b'version')
#         
#                 if data.find('v'):                
#                     message = b'message "./UI/Viewport/Theme/RW_PROG"'
#                     data = sendSocketMsg('localhost', port, message)
#                     if data.find('Ok'):
#                         print("\n\n***Interactive render stopped!****\n\n")
#                         frame = context.scene.frame_current
#                         frameS= "0000" + str(frame)
#                         xmlFilename = currentBlendFile.replace('.blend', '.xml')
#                         os.chdir(os.path.dirname(theaPath))
#                         exporter=initExporter() 
#                         #os.chdir(context.scene.render.filepath)
#                         os.chdir(exportPath)
#                         self.region = context.region
#                         area = context.area
# #                         context.scene['thea_Reuse'] = True 
#                         args = exportFrame(context.scene,context.scene.frame_current,exporter=exporter, area=area)
# #                         context.scene['thea_Reuse'] = False
# #                         message = ('message "Merge %s 4 1 2 1 1 4"' % os.path.join(exportPath, os.path.basename(xmlFilename))).encode()
#                         message = ('message "Load %s"' % os.path.join(exportPath, os.path.basename(xmlFilename))).encode()
# #                         message = ('message "Load %s"' % os.path.join(exportPath, os.path.basename(xmlFilename))).encode() 
#                         print("message: ", message)
#                         data = sendSocketMsg('localhost', port, message)
#                         print("data: ", data)
#                         if data.find('Ok'):
#                             print("Scene merged")
#                             if context.scene.get('thea_IRResolution'):
#                                 if context.scene.get('thea_IRResolution') == 0:
#                                     self.resolutionDivider = 1
#                                 if context.scene.get('thea_IRResolution') == 1:
#                                     self.resolutionDivider = 2
#                                 if context.scene.get('thea_IRResolution') == 2:
#                                     self.resolutionDivider = 4      
#                             #switch to camera frame                            
#                             message = b'message "./UI/Viewport/Theme/RW_CMFR"'                                                
#                             data = sendSocketMsg('localhost', port, message)
#                             print("message: ",message)
#                             print("out: ", data)
#                             if thea_globals.frame_px:
#                                 camM = context.scene.camera.matrix_world
#                                 x1 = thea_globals.frame_px[3][0]
#                                 y2 = thea_globals.frame_px[3][1]
#                                 x2 = thea_globals.frame_px[0][0]
#                                 y1 = thea_globals.frame_px[1][1]       
#                                 resX = int((x2-x1)/self.resolutionDivider)
#                                 resY = int((y2-y1)/self.resolutionDivider)
#                                 message = 'set "./UI/Viewport/Camera/Active" = "%s"' % (bpy.context.scene.camera.name)
#                             else:
#                                 camM = area.spaces.active.region_3d.view_matrix.inverted()
#                                 resX = int(context.region.width/self.resolutionDivider)
#                                 resY = int(context.region.height/self.resolutionDivider)
#                                 #message = 'set "./Cameras/Active" = "%s"' % ("3D View")
#                                 message = 'set "./UI/Viewport/Camera/Active" = "%s"' % ("IR view")
#                             data = sendSocketMsg('localhost', port, message.encode())
#                             print("message: ",message, data)
#                             
#                             message1 = 'set "./UI/Viewport/Camera/Frame" = "%s %s %s %s %s %s %s %s %s %s %s %s"' % (camM[0][0], -camM[0][1], -camM[0][2], camM[0][3], 
#                                                                                                                      camM[1][0], -camM[1][1], -camM[1][2], camM[1][3], 
#                                                                                                                      camM[2][0], -camM[2][1], -camM[2][2], camM[2][3])
#                             data = sendSocketMsg('localhost', port, message1.encode())
#                             #set resolution
#                             if getattr(context.scene, "thea_IRShowTheaWindow", False) == False:
#                                 message = 'set "./UI/Viewport/Camera/Resolution" = "%sx%s"' % (int(context.region.width/self.resolutionDivider),int(context.region.height/self.resolutionDivider))
#                                 print("message: ",message)
#                                 data = sendSocketMsg('localhost', port, message.encode())                        
#                                 print("out: ", data)
#                             message = b'message "./UI/Viewport/Theme/RW_EARF"'
#                             data = sendSocketMsg('localhost', port, message)
#                             if data.find('Ok'):
#                                 print("\n\n***Interactive render started!****\n\n")                                    
#                                 return {'FINISHED'}
#                 elif data == b'ERROR':
#                     self.report({'WARNING'}, "Thea socket server is not running, can't run operator")
#                     return {'CANCELLED'}
#             else:
#                 self.report({'WARNING'}, "IR is not currently running")
#                 return {'CANCELLED'}
#         else:
#             self.report({'WARNING'}, "View3D not found, can't run operator")
#             return {'CANCELLED'}

def thea_sceneUpdated(context):
    from . import thea_globals
    
    print("scene updated")

def thea_collectUpdatedObjects(context):
    from . import thea_globals
    
    thea_globals.sceneUpdated = True

    for ob in bpy.data.objects:
        if ob.is_updated:
            if ob not in thea_globals.objectsUpdated:
                thea_globals.objectsUpdated.append(ob)       
        
def thea_updateIR(con):
     from . import thea_globals
     
     
     
     if not thea_globals.IrIsRunning:
         return
     
     obList = thea_globals.objectsUpdated
#      for ob in bpy.data.objects:
#          if ob.is_updated:
#              obList.append(ob)
            
     port = bpy.context.scene.thea_SDKPort        
     if (len(obList)>0 or thea_globals.materialUpdated):# and os.times()[4]-bpy.context.scene.thea_lastIRUpdate > 5: #don't refresh too often
         print("One or more objects were updated!")         
         bpy.context.scene.thea_lastIRUpdate = os.times()[4]            
         if thea_globals.materialUpdated:
             exportMode = "Material"
             thea_globals.materialUpdated = False
         else:
             exportMode = "Full"                
             
        
         #thea_globals.preview3DAlpha = 0
         (exportPath, theaPath, theaDir, dataPath, currentBlendDir, currentBlendFile) = setPaths(bpy.context.scene)
         data = sendSocketMsg('localhost', port, b'version')
         
    
         if data.find('v'):                
             message = b'message "./UI/Viewport/Theme/RW_PROG"'
             data = sendSocketMsg('localhost', port, message)
             if data.find('Ok'):
                 #print("\n\n***Interactive render started!****\n\n")
                 frame = bpy.context.scene.frame_current
                 frameS= "0000" + str(frame)
                 xmlFilename = currentBlendFile.replace('.blend', '.xml')
                 os.chdir(os.path.dirname(theaPath))
                 exporter=initExporter()
                 #os.chdir(bpy.context.scene.render.filepath)
                 os.chdir(exportPath)
                 area = bpy.context.area
                 bpy.context.scene.thea_Reuse = True
                 currExportAnimation = getattr(bpy.context.scene, 'thea_ExportAnimation')
                 if getattr(bpy.context.scene, 'thea_IRExportAnimation', False):
                    bpy.context.scene.thea_ExportAnimation = True
                 else:
                    bpy.context.scene.thea_ExportAnimation = False
                 args = exportFrame(bpy.context.scene,bpy.context.scene.frame_current,exporter=exporter, area=None, obList=obList, exportMode=exportMode)
                 bpy.context.scene.thea_ExportAnimation = currExportAnimation  
                 
                 thea_globals.objectsUpdated = []
                 bpy.context.scene.thea_Reuse = False                    
#                  message = ('message "Merge %s 4 1 0 1 1 4"' % os.path.join(exportPath, os.path.basename(xmlFilename))).encode() 
                 message = ('message "Merge \'%s\' 0 0 0 0 0 4"' % os.path.join(exportPath, os.path.basename(xmlFilename))).encode()                 
#                  message = ('message "Merge %s 0 0 0 0 0 2"' % os.path.join(exportPath, os.path.basename(xmlFilename))).encode()
                 print("message: ", message)
                 #return                 
                 data = sendSocketMsg('localhost', port, message)                 
                 print("data: ", data)                 
                 if data.find('Ok'):
                     print("Scene merged")
                     message = b'message "./UI/Viewport/Theme/RW_EARF"'
                     data = sendSocketMsg('localhost', port, message)
                     if data.find('Ok'):
                         print("\n\n***Interactive render started!****\n\n")          
                         return 
         else:
             return