# ##### BEGIN GPL LICENSE BLOCK #####
#
#  This program is free software; you can redistribute it and/or
#  modify it under the terms of the GNU General Public License
#  as published by the Free Software Foundation; either version 2
#  of the License, or (at your option) any later version.
#
#  This program is distributed in the hope that it will be useful,
#  but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#  GNU General Public License for more details.
#
#  You should have received a copy of the GNU General Public License
#  along with this program; if not, write to the Free Software Foundation,
#  Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
#
# ##### END GPL LICENSE BLOCK #####

from math import *
from random import random
from TheaForBlender.thea_render_main import *
import bpy
import os
import platform
import struct
import subprocess
import sys
import time
from . import thea_globals
from inspect import getcomments
import configparser



FloatProperty= bpy.types.FloatProperty
IntProperty= bpy.types.IntProperty
BoolProperty= bpy.types.BoolProperty
EnumProperty= bpy.types.EnumProperty
CollectionProperty= bpy.types.CollectionProperty
StringProperty= bpy.types.StringProperty

Scene = bpy.types.Scene
Mat = bpy.types.Material
Tex = bpy.types.Texture
Obj = bpy.types.Object
Cam = bpy.types.Camera
WM  = bpy.types.WindowManager
Part = bpy.types.ParticleSettings
Lamp = bpy.types.Lamp

#(exportPath, theaPath, theaDir, dataPath, currentBlendDir, currentBlendFile) = setPaths(bpy.context.scene)

try:
    if bpy.context.scene:
        sceneLoaded = True
except:
    dataPath = ""

Scene.thea_sdkIsRunning = bpy.props.BoolProperty(default= False)
Scene.thea_sdkProcess = bpy.props.IntProperty()

def engineUpdated(self, context):
    from . import thea_globals
    thea_globals.engineUpdated = True

def TheaPathUpdated(self, context, origin=""):    
    print("path: ",  context, origin, context.scene.get(origin),bpy.path.abspath(context.scene.get(origin)))
    config = configparser.ConfigParser()
    configPath = os.path.join(os.path.dirname(os.path.realpath(__file__)), "config.ini")
    config.read(configPath)
    if not config.has_section("PATHS"):
        config.add_section('PATHS')
    config.set('PATHS',origin,os.path.abspath(bpy.path.abspath(context.scene.get(origin))))
    with open(configPath, 'w') as configfile:
        config.write(configfile)    



Scene.thea_ApplicationPath = bpy.props.StringProperty(
                  name = "Thea Application path",
                  default = "",
                  description = "Path to Thea executable",
                  subtype = 'FILE_PATH',
                  update=lambda a,b: TheaPathUpdated(a,b,"thea_ApplicationPath")
                  )      

Scene.thea_DataPath = bpy.props.StringProperty(
                  name = "Thea Data path",
                  default = "",
                  description = "Path to Thea data directory",
                  subtype = 'FILE_PATH',
                  update=lambda a,b: TheaPathUpdated(a,b,"thea_DataPath")
                  )      

Scene.thea_RenderEngineMenu = bpy.props.EnumProperty(   
                items=(("Adaptive (BSD)","Adaptive (BSD)","Adaptive (BSD)"),("Unbiased (TR1)","Unbiased (TR1)","Unbiased (TR1)"),("Unbiased (TR2)","Unbiased (TR2)", "Unbiased (TR2)"),("Adaptive (AMC)","Adaptive (AMC)","Adaptive (AMC)"),("Presto (AO)","Presto (AO)","Presto (AO)"),("Presto (MC)","Presto (MC)","Presto (MC)")),
                #items=(("Adaptive (BSD)","Adaptive (BSD)","Adaptive (BSD)"),("Progressive (BSD)","Progressive (BSD)","Progressive (BSD)"),("Unbiased (TR1)","Unbiased (TR1)","Unbiased (TR1)"),("Unbiased (TR2)","Unbiased (TR2)", "Unbiased (TR2)"),("Unbiased* (MC)","Unbiased* (MC)","Unbiased* (MC)"),("Adaptive (AMC)","Adaptive (AMC)","Adaptive (AMC)"),("Presto (AO)","Presto (AO)","Presto (AO)"),("Presto (MC)","Presto (MC)","Presto (MC)")),
#                items=(("BSD","Adaptive (BSD)","Adaptive (BSD)"),("TR1","Unbiased (TR1)","Unbiased (TR1)"),("TR1+","Unbiased (TR1+)","Unbiased (TR1+)"),("TR2","Unbiased (TR2)", "Unbiased (TR2)"),("TR2+","Unbiased (TR2+)", "Unbiased (TR2+)")),                
                name="Engine Core",
                description="Engine Core",
                default="Presto (AO)")

Scene.thea_IRRenderEngineMenu = bpy.props.EnumProperty(   
                items=(("Presto (AO)","Presto (AO)","Presto (AO)"),("Presto (MC)","Presto (MC)","Presto (MC)"),("Adaptive (AMC)","Adaptive (AMC)","Adaptive (AMC)")),                
                #items=(("Progressive (BSD)","Progressive (BSD)","Progressive (BSD)"),("Adaptive (AMC)","Adaptive (AMC)","Adaptive (AMC)"),("Presto (AO)","Presto (AO)","Presto (AO)"),("Presto (MC)","Presto (MC)","Presto (MC)")),
                name="Engine Core",
                description="Engine Core",
                default="Presto (AO)",
                update=engineUpdated)
                    
Scene.thea_AASamp = bpy.props.EnumProperty(   
                items=(("0","Default","Default"),("1","None","None"),("2","Normal", "Normal"),("3", "High", "High")),
                name="Supersampling",
                description="Supersampling",
                default="0")

Scene.thea_RenderTime = bpy.props.IntProperty(   
                min=0, 
                max=1000,
                default=0,
                name="Time Limit (min)",
                description="Time Limit (min)")
                
Scene.thea_RenderMaxPasses = bpy.props.IntProperty(   
                min=1, 
                max=1000000,
                default=10000,
                name="Max Passes",
                description="Max Passes")                

Scene.thea_RenderMaxSamples = bpy.props.IntProperty(   
                min=1, 
                max=1000000,
                default=512,
                name="Max Samples",
                description="Max Samples")

Scene.thea_RenderMBlur = bpy.props.BoolProperty(   
                name="Motion Blur",
                description="Motion Blur",
                default= True)

Scene.thea_RenderVolS = bpy.props.BoolProperty(   
                name="Volumetric Scattering",
                description="Volumetric Scattering",
                default= True)

Scene.thea_RenderLightBl = bpy.props.BoolProperty(   
                name="Relight",
                description="Relight",
                default= False) 

Scene.thea_RenderRefreshResult = bpy.props.BoolProperty(   
                name="Auto Refresh result",
                description="Auto refresh render result",
                default= True)

Scene.thea_ExportAnimation = bpy.props.BoolProperty(   
                name="Export Animation data of objects",
                description="Export Animation data of objects",
                default= True)

Scene.thea_AnimationEveryFrame = bpy.props.BoolProperty(   
                name="Export every animation key frame",
                description="Export every animation key frame",
                default= False)

Scene.thea_Reuse = bpy.props.BoolProperty(   
                name="Reuse Meshes",
                description="Reuse already exported meshes",
                default= False)   

Scene.thea_ExitAfterRender = bpy.props.BoolProperty(   
                name="Exit Thea after render",
                description="Exit Thea after render",
                default= False) 

Scene.thea_stopOnErrors = bpy.props.BoolProperty(   
                name="Stop on errors",
                description="Stop on errors",
                default= True)                 
                

Scene.thea_Selected = bpy.props.BoolProperty(   
                name="Selected only",
                description="Export only selected objects",
                default= False)   

Scene.thea_HideThea = bpy.props.BoolProperty(   
                name="Hide Thea while rendering",
                description="Hide Thea while rendering",
                default= True)                   

Scene.thea_RenderLightBl = bpy.props.BoolProperty(   
                name="Relight",
                description="Relight",
                default= False)   

Scene.thea_DistTh = bpy.props.EnumProperty(   
                items=(("0", "Max", "Max"),("1","1","1"),("2","2","2"),("4","4", "4"),("8", "8", "8")),
                name="Threads",
                description="Threads",
                default="0")

Scene.thea_DistPri = bpy.props.EnumProperty(   
                items=(("0","Low","Low"),("1","Normal","Normal")),
                name="Priority",
                description="Priority",
                default="0")

Scene.thea_DistNet = bpy.props.EnumProperty(   
                items=(("0","None","None"),("1","Client","Client"), ("2", "Server", "Server")),
                name="Network",
                description="Network",
                default="0")

Scene.thea_DistPort = bpy.props.IntProperty(   
                min=1001, 
                max=65000,
                default=6200,
                name="Server Port",
                description="Server Port")

Scene.thea_DistAddr = bpy.props.StringProperty( 
                maxlen=15,
                default="127.0.0.1",
                name="Server Address",
                description="Server Address")

Scene.thea_BucketRendering = bpy.props.BoolProperty(   
                name="Bucket rendering",
                description="Bucket rendering",
                default= False)   

Scene.thea_ExtendedTracing = bpy.props.BoolProperty(   
                name="Extended Tracing",
                description="Extended Tracing",
                default= False)

Scene.thea_TransparencyDepth = bpy.props.IntProperty(   
                min=0, 
                max=100,
                default=15,
                name="Transparency Depth",
                description="Transparency Depth")

Scene.thea_InternalReflectionDepth = bpy.props.IntProperty(   
                min=0, 
                max=100,
                default=15,
                name="Internal Reflection Depth",
                description="Internal Reflection Depth")

Scene.thea_SSSDepth = bpy.props.IntProperty(   
                min=0, 
                max=100,
                default=15,
                name="SSS Depth",
                description="SSS Depth")

Scene.thea_RTTracingDepth = bpy.props.IntProperty(   
                min=0, 
                max=100,
                default=5,
                name="Tracing Depth",
                description="Tracing Depth")

Scene.thea_RTDiffuseDepth = bpy.props.IntProperty(   
                min=0, 
                max=100,
                default=1,
                name="Diffuse Depth",
                description="Diffuse Depth")

Scene.thea_RTGlossyDepth = bpy.props.IntProperty(   
                min=0, 
                max=100,
                default=2,
                name="Glossy Depth",
                description="Glossy Depth")

Scene.thea_RTTraceDispersion = bpy.props.BoolProperty(   
                name="Trace Dispersion",
                description="Trace Dispersion",
                default= False)   

Scene.thea_RTTraceReflections = bpy.props.BoolProperty(   
                name="Trace Reflections",
                description="Trace Reflections",
                default= True)   

Scene.thea_RTTraceRefractions = bpy.props.BoolProperty(   
                name="Trace Refractions",
                description="Trace Refractions",
                default= True)   

Scene.thea_RTTraceTransparencies = bpy.props.BoolProperty(   
                name="Trace Transparencies",
                description="Trace Transparencies",
                default= True)   

Scene.thea_AACont = bpy.props.FloatProperty(  
                min=0.000, 
                max=10.000,
                precision=3,
                default=0.020,
                name="Max Contrast",
                description="Max Contrast")

Scene.thea_AAMinSub = bpy.props.EnumProperty(   
                items=(("0","0","0"),("1","1","1"), ("2", "2", "2"), ("3", "3", "3"), ("4", "4", "4"), ("5", "5", "5"), ("6", "6", "6"), ("7", "7", "7")),
                name="Min AA Subdivs",
                description="Min AA Subdivs",
                default="0")

Scene.thea_AAMaxSub = bpy.props.EnumProperty(   
                items=(("0","0","0"),("1","1","1"), ("2", "2", "2"), ("3", "3", "3"), ("4", "4", "4"), ("5", "5", "5"), ("6", "6", "6"), ("7", "7", "7")),
                name="Max AA Subdivs",
                description="Max AA Subdivs",
                default="4")

Scene.thea_DLEnable = bpy.props.BoolProperty(   
                name="Enable",
                description="Enable",
                default= True) 

Scene.thea_DLPerceptualBased = bpy.props.BoolProperty(   
                name="Perceptual Based",
                description="Perceptual Based",
                default= False) 

Scene.thea_DLMaxError = bpy.props.FloatProperty(  
                min=0.000, 
                max=100,
                precision=3,
                default=2,
                name="Max Direct Error %",
                description="Max Direct Error %")

Scene.thea_BREnable = bpy.props.BoolProperty(   
                name="Enable",
                description="Enable",
                default= True) 

Scene.thea_BRMinSub = bpy.props.EnumProperty(   
                items=(("0","0","0"),("1","1","1"), ("2", "2", "2"), ("3", "3", "3"), ("4", "4", "4"), ("5", "5", "5"), ("6", "6", "6"), ("6", "6", "6")),
                name="Min Blurred Subdivs",
                description="Min Blurred Subdivs",
                default="0")

Scene.thea_BRMaxSub = bpy.props.EnumProperty(   
                items=(("0","0","0"),("1","1","1"), ("2", "2", "2"), ("3", "3", "3"), ("4", "4", "4"), ("5", "5", "5"), ("6", "6", "6"), ("6", "6", "6")),
                name="Max Blurred Subdivs",
                description="Max Blurred Subdivs",
                default="3")

Scene.thea_GIEnable = bpy.props.BoolProperty(   
                name="Enable",
                description="Enable",
                default= False)

Scene.thea_FMEnable = bpy.props.BoolProperty(   
                name="Enable",
                description="Enable",
                default= False)

Scene.thea_FMFieldDensity = bpy.props.IntProperty(   
                min=0, 
                max=1000000000,
                default=100000,
                name="Field Density",
                description="Field Density")

Scene.thea_FMCellSize = bpy.props.IntProperty(   
                min=0, 
                max=1000000000,
                default=1000,
                name="Cell Size",
                description="Cell Size")

Scene.thea_GICaustics = bpy.props.BoolProperty(   
                name="Caustics",
                description="Caustics",
                default= False)

Scene.thea_GITotPh = bpy.props.IntProperty(   
                min=0, 
                max=1000000000,
                default=1000000,
                name="Photons Captured",
                description="Photons Captured")

Scene.thea_GICausticCaptured = bpy.props.IntProperty(   
                min=0, 
                max=1000000000,
                default=10000,
                name="Captured Photons",
                description="Captured Photons")

Scene.thea_GICausPh = bpy.props.IntProperty(   
                min=0, 
                max=1000,
                default=200,
                name="Estimation Photons",
                description="Estimation Photons")

Scene.thea_GILeakComp = bpy.props.BoolProperty(   
                name="Leak Compensation",
                description="Leak Compensation",
                default= False)

Scene.thea_GICausticSharp = bpy.props.BoolProperty(   
                name="Caustic Sharpening",
                description="Caustic Sharpening",
                default= False)

Scene.thea_GIGlobPh = bpy.props.IntProperty(   
                min=0, 
                max=1000000,
                default=400,
                name="Global Photons",
                description="Global Photons")

Scene.thea_GIGlobalRadius = bpy.props.FloatProperty(  
                min=0.0001, 
                max=1000,
                precision=4,
                default=0.50,
                name="Global Radius",
                description="Global Radius")

Scene.thea_GICausticRadius = bpy.props.FloatProperty(  
                min=0.000, 
                max=100,
                precision=4,
                default=0.10,
                name="Caustic Radius",
                description="Caustic Radius")

Scene.thea_CausticLock = bpy.props.BoolProperty(   
                name="Lock Cache",
                description="Lock Cache",
                default= False)

Scene.thea_FGEnable = bpy.props.BoolProperty(   
                name="Enable",
                description="Enable",
                default= True)

Scene.thea_FGDiffuseDepth = bpy.props.IntProperty(   
                min=0, 
                max=100,
                default=1,
                name="Diffuse Depth",
                description="Diffuse Depth")

Scene.thea_GIRays = bpy.props.IntProperty(   
                min=1, 
                max=10000,
                default=200,
                name="Samples",
                description="Samples")

Scene.thea_GIError = bpy.props.FloatProperty(  
                min=0.01, 
                max=100,
                precision=3,
                default=15,
                name="Max Gather Error (%)",
                subtype='PERCENTAGE',
                description="Max Gather Error (%)")

Scene.thea_FGEnableSecondary = bpy.props.BoolProperty(   
                name="Secondary Gather",
                description="Secondary Gather",
                default= False)

Scene.thea_FGDistanceThreshold = bpy.props.FloatProperty(  
                min=0.001, 
                max=100,
                precision=3,
                default=0.100,
                name="Distance Threshold (m)",
                description="Distance Threshold (m)")

Scene.thea_GITracingDepth = bpy.props.IntProperty(   
                min=1, 
                max=100,
                default=1,
                name="Tracing Depth",
                description="Tracing Depth")

Scene.thea_FGGlossyDepth = bpy.props.IntProperty(
                min=0, 
                max=100,
                default=0,
                name="Glossy Depth",
                description="Glossy Depth")

Scene.thea_FGAdaptiveSteps = bpy.props.IntProperty(   
                min=0, 
                max=100,
                default=3,
                name="Adaptive Steps",
                description="Adaptive Steps")

Scene.thea_FGGlossyEvaluation = bpy.props.BoolProperty(   
                name="Glossy Evaluation",
                description="Glossy Evaluation",
                default= False)   

Scene.thea_FGCausticEvaluation = bpy.props.BoolProperty(   
                name="Caustic Evaluation",
                description="Caustic Evaluation",
                default= False)   

Scene.thea_GITraceReflections = bpy.props.BoolProperty(   
                name="Trace Reflections",
                description="Trace Reflections",
                default= True)   

Scene.thea_GITraceRefractions = bpy.props.BoolProperty(   
                name="Trace Refractions",
                description="Trace Refractions",
                default= True)   

Scene.thea_GITraceTransparencies = bpy.props.BoolProperty(   
                name="Trace Transparencies",
                description="Trace Transparencies",
                default= True)   

Scene.thea_GIEnableSS = bpy.props.BoolProperty(   
                name="Enable",
                description="Enable",
                default= False)

Scene.thea_UseZip = bpy.props.BoolProperty(
                name="Compress",
                description="Compress",
                default= False)


def worldUpdated(self, context):
    from . import thea_globals
    thea_globals.worldUpdated = True
    
def worldFilenameUpdated(self, context, origin=""):    
    imgName = context.scene.get(origin)    
    texName = "IBL_"+origin
    world = context.scene.world
    print("worldFilenameUpdated: ",self, context, origin, context.scene.get(origin))
    exists = False
    try:
        if world.texture_slots[texName]:
            exists = True
            slot = world.texture_slots[texName]
            tex = slot.texture                       
    except:
        pass
            
    if exists:
        try:
            if imgName:
                img = bpy.data.images.load(imgName)
                tex.image = img
            else:
                print("removing texture: ", slot, tex)
                world.texture_slots[texName].texture = None
        except:
            pass
    else:
        img = bpy.data.images.load(imgName)
        tex = bpy.data.textures.new(name=texName, type='IMAGE') 
        tex.image = img
        tex.name = texName
        slot = world.texture_slots.add()
        slot.texture = tex
    

Scene.thea_IBLEnable = bpy.props.BoolProperty(   
                name="Enable Image Based Lighting",
                description="Enable Image Based Lighting",
                default= False,
                update=worldUpdated)

Scene.thea_IBLTypeMenu = bpy.props.EnumProperty(   
                items=(("IBL Only","IBL Only","IBL Only"),("Sun+IBL","Sun+IBL","Sun+IBL")),                               
                name="IBL Type",
                description="IBL type",
                default="IBL Only",
                update=worldUpdated)

Scene.thea_IBLWrappingMenu = bpy.props.EnumProperty(   
                items=(("AngularProbe","AngularProbe","AngularProbe"),("Hemispherical","Hemispherical","Hemispherical"), ("Spherical","Spherical","Spherical")),                               
                name="Wrapping",
                description="Wrapping",
                default="Spherical",
                update=worldUpdated)

Scene.thea_BackroundWrappingMenu = bpy.props.EnumProperty(   
                items=(("AngularProbe","AngularProbe","AngularProbe"),("Hemispherical","Hemispherical","Hemispherical"), ("Spherical","Spherical","Spherical")),                               
                name="Wrapping",
                description="Wrapping",
                default="Spherical",
                update=worldUpdated)



Scene.thea_IBLFilename = bpy.props.StringProperty(
                  name = "Filename",
                  default = "",
                  description = "IBL file path",
                  subtype = 'FILE_PATH',
                  update=lambda a,b: worldFilenameUpdated(a,b,"thea_IBLFilename")
                  )

Scene.thea_IBLRotation = bpy.props.FloatProperty(  
                min=-10000000.00, 
                max=10000000,
                precision=2,
                default=0,
                name="Rotation (deg)",
                description="Rotation (deg)",
                update=worldUpdated)

Scene.thea_IBLIntensity = bpy.props.FloatProperty(  
                min=0.00, 
                max=10000000,
                precision=2,
                default=1,
                name="Intensity",
                description="Intensity",
                update=worldUpdated)

Scene.thea_BackgroundMappingEnable = bpy.props.BoolProperty(   
                name="Enable Background Mapping",
                description="Enable Background Mapping",
                default= False,
                update=worldUpdated)

Scene.thea_BackgroundMappingTypeMenu = bpy.props.EnumProperty(   
                items=(("IBL Only","IBL Only","IBL Only"),("Sun+IBL","Sun+IBL","Sun+IBL")),                               
                name="IBL Type",
                description="IBL type",
                default="IBL Only",
                update=worldUpdated)

Scene.thea_BackgroundMappingWrappingMenu = bpy.props.EnumProperty(   
                items=(("AngularProbe","AngularProbe","AngularProbe"),("Hemispherical","Hemispherical","Hemispherical"), ("Spherical","Spherical","Spherical"), ("Center", "Center", "Center"), ("Tile", "Tile", "Tile"), ("Fit", "Fit", "Fit")),                               
                name="Wrapping",
                description="Wrapping",
                default="Spherical",
                update=worldUpdated)

Scene.thea_BackgroundMappingFilename = bpy.props.StringProperty(   
                name="Filename",
                description="Filename",
                default= "",
                subtype = 'FILE_PATH',
                  update=lambda a,b: worldFilenameUpdated(a,b,"thea_BackgroundMappingFilename"))

Scene.thea_BackgroundMappingRotation = bpy.props.FloatProperty(  
                min=-10000000.00, 
                max=10000000,
                precision=2,
                default=0,
                name="Rotation (deg)",
                description="Rotation (deg)",
                update=worldUpdated)

Scene.thea_BackgroundMappingIntensity = bpy.props.FloatProperty(  
                min=0.00, 
                max=10000000,
                precision=2,
                default=1,
                name="Intensity",
                description="Intensity",
                update=worldUpdated)

Scene.thea_ReflectionMappingEnable = bpy.props.BoolProperty(   
                name="Enable Reflection Mapping",
                description="Enable Reflection Mapping",
                default= False,
                update=worldUpdated)

Scene.thea_ReflectionMappingTypeMenu = bpy.props.EnumProperty(   
                items=(("IBL Only","IBL Only","IBL Only"),("Sun+IBL","Sun+IBL","Sun+IBL")),                               
                name="IBL Type",
                description="IBL type",
                default="IBL Only",
                update=worldUpdated)

Scene.thea_ReflectionMappingWrappingMenu = bpy.props.EnumProperty(   
                items=(("AngularProbe","AngularProbe","AngularProbe"),("Hemispherical","Hemispherical","Hemispherical"), ("Spherical","Spherical","Spherical")),                               
                name="Wrapping",
                description="Wrapping",
                default="Spherical",
                update=worldUpdated)

Scene.thea_ReflectionMappingFilename = bpy.props.StringProperty(   
                name="Filename",
                description="Filename",
                default= "",
                subtype = 'FILE_PATH',
                  update=lambda a,b: worldFilenameUpdated(a,b,"thea_ReflectionMappingFilename"))

Scene.thea_ReflectionMappingRotation = bpy.props.FloatProperty(  
                min=-10000000.00, 
                max=10000000,
                precision=2,
                default=0,
                name="Rotation (deg)",
                description="Rotation (deg)",
                update=worldUpdated)

Scene.thea_ReflectionMappingIntensity = bpy.props.FloatProperty(  
                min=0.00, 
                max=10000000,
                precision=2,
                default=1,
                name="Intensity",
                description="Intensity",
                update=worldUpdated)

Scene.thea_RefractionMappingEnable = bpy.props.BoolProperty(   
                name="Enable Refraction Mapping",
                description="Enable Refraction Mapping",
                default= False,
                update=worldUpdated)

Scene.thea_RefractionMappingTypeMenu = bpy.props.EnumProperty(   
                items=(("IBL Only","IBL Only","IBL Only"),("Sun+IBL","Sun+IBL","Sun+IBL")),                               
                name="IBL Type",
                description="IBL type",
                default="IBL Only",
                update=worldUpdated)

Scene.thea_RefractionMappingWrappingMenu = bpy.props.EnumProperty(   
                items=(("AngularProbe","AngularProbe","AngularProbe"),("Hemispherical","Hemispherical","Hemispherical"), ("Spherical","Spherical","Spherical")),                               
                name="Wrapping",
                description="Wrapping",
                default="Spherical",
                update=worldUpdated)

Scene.thea_RefractionMappingFilename = bpy.props.StringProperty(   
                name="Filename",
                description="Filename",
                default= "",
                subtype = 'FILE_PATH',
                  update=lambda a,b: worldFilenameUpdated(a,b,"thea_RefractionMappingFilename"))

Scene.thea_RefractionMappingRotation = bpy.props.FloatProperty(  
                min=-10000000.00, 
                max=10000000,
                precision=2,
                default=0,
                name="Rotation (deg)",
                description="Rotation (deg)",
                update=worldUpdated)

Scene.thea_RefractionMappingIntensity = bpy.props.FloatProperty(  
                min=0.00, 
                max=10000000,
                precision=2,
                default=1,
                name="Intensity",
                description="Intensity",
                update=worldUpdated)

Scene.thea_IBLEnable = bpy.props.BoolProperty(   
                name="Enable Image Based Lighting",
                description="Enable Image Based Lighting",
                default= False,
                update=worldUpdated)

Scene.thea_IBLTypeMenu = bpy.props.EnumProperty(   
                items=(("IBL Only","IBL Only","IBL Only"),("Sun+IBL","Sun+IBL","Sun+IBL")),                               
                name="IBL Type",
                description="IBL type",
                default="IBL Only",
                update=worldUpdated)

Scene.thea_IBLWrappingMenu = bpy.props.EnumProperty(   
                items=(("AngularProbe","AngularProbe","AngularProbe"),("Hemispherical","Hemispherical","Hemispherical"), ("Spherical","Spherical","Spherical")),                               
                name="Wrapping",
                description="Wrapping",
                default="Spherical",
                update=worldUpdated)

Scene.thea_IBLFilePath = bpy.props.StringProperty(   
                name="Filename",
                description="Filename",
                default= "")

Scene.thea_IBLRotation = bpy.props.FloatProperty(  
                min=-10000000.00, 
                max=10000000,
                precision=2,
                default=0,
                name="Rotation (deg)",
                description="Rotation (deg)",
                update=worldUpdated)

Scene.thea_IBLIntensity = bpy.props.FloatProperty(  
                min=0.00, 
                max=10000000,
                precision=2,
                default=1,
                name="Intensity",
                description="Intensity",
                update=worldUpdated)

Scene.thea_EnvPSEnable = bpy.props.BoolProperty(   
                name="Enable Physical Sky",
                description="Enable Physical Sky",
                default= False,
                update=worldUpdated)

Scene.thea_EnvPSTurb = bpy.props.FloatProperty(  
                min=0.00, 
                max=100,
                precision=3,
                default=2.500,
                name="Turbidity",
                description="Turbidity",
                update=worldUpdated)

Scene.thea_EnvPSOzone = bpy.props.FloatProperty(  
                min=0.00, 
                max=100,
                precision=3,
                default=0.350,
                name="Ozone",
                description="Ozone",
                update=worldUpdated)

Scene.thea_EnvPSWatVap = bpy.props.FloatProperty(  
                min=0.00, 
                max=100,
                precision=3,
                default=2.000,
                name="Water Vapor",
                description="Water Vapor",
                update=worldUpdated)

Scene.thea_EnvPSTurbCo = bpy.props.FloatProperty(  
                min=0.00, 
                max=1,
                precision=3,
                default=0.046,
                name="Turbidity Coefficient",
                description="Turbidity Coefficient",
                update=worldUpdated)

Scene.thea_EnvPSWaveExp = bpy.props.FloatProperty(  
                min=0.00, 
                max=100,
                precision=3,
                default=1.300,
                name="Wavelenght Exponent",
                description="Wavelenght Exponent",
                update=worldUpdated)

def locationUpdated(self, context):
    scene = context.scene       
    loc = getLocation(scene.thea_EnvLocationsMenu, getLocations()[1], scene)
    if loc[0] != "":
        scene.thea_EnvLat = loc[0]
        scene.thea_EnvLong = loc[1]
        scene.thea_EnvTZ = str(loc[2])


Scene.thea_EnvLocationsMenu = bpy.props.EnumProperty(   
                items=getLocations()[0],
                name="Location",
                description="Location",
                default="3",
                update=locationUpdated)
                
Scene.thea_Interface = bpy.props.EnumProperty(   
                 name="Materials List",
                 description="Materials List",
                 items=(("0","None","0"),("0","None","0")),
                 default="0")                


Scene.thea_EnvLat = bpy.props.StringProperty( 
                maxlen=15,
                default="0",
                name="Latitude",
                description="Latitude",
                update=worldUpdated)

Scene.thea_EnvLong = bpy.props.StringProperty( 
                maxlen=15,
                default="0",
                name="Longitude",
                description="Longitude",
                update=worldUpdated)

Scene.thea_EnvTZ = bpy.props.EnumProperty(   
                items=(('-12','GT-12',''),('-11','GT-11',''),('-10','GT-10',''),('-9','GT-09',''),('-8','GT-08',''),('-7','GT-07',''),('-6','GT-06',''),('-5','GT-05',''),('-4','GT-04',''),('-3','GT-03',''),('-2','GT-02',''),('-1','GT-01',''),('0','GT+00',''),('1','GT+01',''),('2','GT+02',''),('3','GT+03',''),('4','GT+04',''),('5','GT+05',''),('6','GT+06',''),('7','GT+07',''),('8','GT+08',''),('9','GT+09',''),('10','GT+10',''),('11','GT+11',''),('12','GT+12','')),
                name="Timezone",
                description="Timezone",
                default="0",
                update=worldUpdated)

Scene.thea_EnvTime = bpy.props.StringProperty( 
                maxlen=8,
                default="12:00:00",
                name="Time",
                description="Time",
                update=worldUpdated)

Scene.thea_EnvDate = bpy.props.StringProperty( 
                maxlen=10,
                default="1/6/2010",
                name="Date",
                description="Date",
                update=worldUpdated)

Scene.thea_EnvUpdateSky = bpy.props.BoolProperty(   
                name="Update Sky",
                description="Update Sky",
                default= True)

def displayUpdated(self, context):
    thea_globals.displayUpdated = True
    
Scene.thea_DispISO = bpy.props.IntProperty(   
                min=1, 
                max=1000000,
                default=100,
                name="ISO",
                description="ISO",
                update=displayUpdated)

Scene.thea_DispShutter = bpy.props.IntProperty(   
                min=1, 
                max=1000000,
                default=250,
                name="Shutter Speed",
                description="Shutter Speed",
                update=displayUpdated)

Scene.thea_DispFNumber = bpy.props.FloatProperty(  
                min=0.1, 
                max=100,
                precision=1,
                default=5.6,
                name="f-number",
                description="f-number",
                update=displayUpdated)

Scene.thea_DispGamma = bpy.props.FloatProperty(  
                min=0.1, 
                max=10,
                precision=3,
                default=2.200,
                name="Gamma",
                description="Gamma",
                update=displayUpdated)

Scene.thea_DispBrightness = bpy.props.FloatProperty(  
                min=0.1, 
                max=100,
                precision=3,
                default=1.000,
                name="Brightness",
                description="Brightness",
                update=displayUpdated)

Scene.thea_DispContrast = bpy.props.BoolProperty(   
                name="Contrast",
                description="Contrast",
                default= False,
                update=displayUpdated)

Scene.thea_DispContrastWeight = bpy.props.FloatProperty(  
                min=0.0, 
                max=100,
                precision=3,
                default=0.000,
                name="Contrast",
                description="Contrast",
                subtype='PERCENTAGE',
                update=displayUpdated)

Scene.thea_DispChroma = bpy.props.BoolProperty(   
                name="Chroma",
                description="Chroma",
                default= False,                
                update=displayUpdated)

Scene.thea_DispChromaWeight = bpy.props.FloatProperty(  
                min=0.0, 
                max=100,
                precision=3,
                default=0.000,
                subtype='PERCENTAGE',
                name="Chroma",
                description="Chroma",
                update=displayUpdated)

Scene.thea_DispTemperature = bpy.props.BoolProperty(   
                name="Temperature",
                description="Temperature",
                default= False,
                update=displayUpdated)

Scene.thea_DispTemperatureWeight = bpy.props.FloatProperty(  
                min=0.1, 
                max=100000,
                precision=1,
                default=6500,
                name="White Balance",
                description="White Balance",
                update=displayUpdated)


Scene.thea_MerModels = bpy.props.EnumProperty(   
                items=(('0', 'Keep Current - Throw Away New', ''),('1', 'Replace With New - Throw Away Current', ''), ('2', 'Merge - Add New To Current',''),('3', 'Merge - Name Collision: Keep', ''), ('4', 'Merge - Name Collision: Replace', ''), ('5', 'Merge - Name Collision: Replace Mesh', ''), ('6', 'Merge - Name Collision: Replace Material', '')),
                name="Models",
                description="How to merge models",
                default="0")

Scene.thea_MerLights = bpy.props.EnumProperty(   
                items=(('0', 'Keep Current - Throw Away New', ''),('1', 'Replace With New - Throw Away Current', ''), ('2', 'Merge - Add New To Current',''),('3', 'Merge - Name Collision: Keep', ''), ('4', 'Merge - Name Collision: Replace', ''), ('5', 'Merge - Name Collision: Replace Possition', ''), ('6', 'Merge - Name Collision: Replace Emitter', '')),
                name="Lights",
                description="How to merge lights",
                default="0")

Scene.thea_MerCameras = bpy.props.EnumProperty(   
                items=(('0', 'Keep Current - Throw Away New', ''),('1', 'Replace With New - Throw Away Current', ''), ('2', 'Merge - Add New To Current',''),('3', 'Merge - Name Collision: Keep', ''), ('4', 'Merge - Name Collision: Replace', ''), ('5', 'Merge - Name Collision: Replace Mesh', ''), ('6', 'Merge - Name Collision: Replace Lens', '')),
                name="Cameras",
                description="How to merge cameras",
                default="0")

Scene.thea_MerEnv = bpy.props.EnumProperty(   
                items=(('0', 'Keep Current - Throw Away New', ''),('1', 'Replace With New - Throw Away Current', '')),
                name="Environment",
                description="How to merge environment",
                default="0")

Scene.thea_MerRender = bpy.props.EnumProperty(   
                items=(('0', 'Keep Current - Throw Away New', ''),('1', 'Replace With New - Throw Away Current', '')),
                name="Render",
                description="How to merge render settings",
                default="0") 

Scene.thea_MerMaterials = bpy.props.EnumProperty(   
                items=(('0', 'Add new to Current', ''),('1', 'Merge - Name Collision: Keep', ''), ('2', 'Merge - Name Collision: Replace', '')),
                name="Materials",
                description="How to merge materials",
                default="0")

Scene.thea_MerSurfaces = bpy.props.EnumProperty(   
                items=(('0', 'Add new to Current', ''),('1', 'Merge - Name Collision: Keep', ''), ('2', 'Merge - Name Collision: Replace', '')),
                name="Surfaces",
                description="How to merge surfaces",
                default="0")

Scene.thea_ICEnable = bpy.props.BoolProperty(   
                name="Enable",
                description="Enable",
                default= True)

Scene.thea_ICLock = bpy.props.BoolProperty(   
                name="Lock Cache",
                description="Lock Cache",
                default= False)

Scene.thea_ICAccuracy = bpy.props.FloatProperty(  
                min=0, 
                max=100,
                precision=3,
                default=85,
                name="Sample Density",
                description="Sample Density")

Scene.thea_ICMinDistance = bpy.props.FloatProperty(  
                min=0, 
                max=100,
                precision=3,
                default=0.2,
                name="Min Distance",
                description="Min Distance")

Scene.thea_ICMaxDistance = bpy.props.FloatProperty(  
                min=0, 
                max=100,
                precision=3,
                default=10,
                name="Max Distance",
                description="Max Distance")

Scene.thea_ICMinDistance = bpy.props.FloatProperty(  
                min=0, 
                max=100,
                precision=3,
                default=4,
                name="Min Distance",
                description="Min Distance")

Scene.thea_ICPrepass = bpy.props.EnumProperty(   
                items=(('0', 'None', ''),('1', '1/1', ''),('2', '1/2', ''),('3', '1/4', ''),('4', '1/8', '')),
                name="Prepass",
                description="Prepass",
                default="1")

Scene.thea_ICPrepassBoost = bpy.props.FloatProperty(  
                min=0, 
                max=10,
                precision=3,
                default=0.5,
                name="Prepass Boost",
                description="Prepass Boost")

Scene.thea_ICForceInterpolation = bpy.props.BoolProperty(   
                name="ForceInterpolation",
                description="ForceInterpolation",
                default= True)

Scene.thea_Warning = bpy.props.StringProperty( 
                maxlen=15,
                default="0",
                name="Warning",
                description="Warning")

Scene.thea_RenderPresetsMenu = bpy.props.EnumProperty(   
                items=getTheaPresets(),
                name="Render Presets",
                description="Render Presets",
                default="0")                

Scene.thea_maxLines = bpy.props.IntProperty(   
                min=200, 
                max=2000,
                default=200,
                name="maxLines",
                description="maxLines")

Scene.thea_dataPath = bpy.props.StringProperty( 
                maxlen=1024,
                default="",
                name="Thea data path",
                description="Thea data path")

Scene.thea_channelNormal = bpy.props.BoolProperty(   
                name="Normal",
                description="Normal",
                default= False)

Scene.thea_channelPosition = bpy.props.BoolProperty(   
                name="Position",
                description="Position",
                default= False)

Scene.thea_channelUV = bpy.props.BoolProperty(   
                name="UV",
                description="UV",
                default= False)

Scene.thea_channelDepth = bpy.props.BoolProperty(   
                name="Depth",
                description="Depth",
                default= False)

Scene.thea_channelAlpha = bpy.props.BoolProperty(   
                name="Alpha",
                description="Alpha",
                default= False)

Scene.thea_channelObjectId = bpy.props.BoolProperty(   
                name="Object Id",
                description="Object Id",
                default= False)

Scene.thea_channelMaterialId = bpy.props.BoolProperty(   
                name="Material Id",
                description="Material Id",
                default= False)

Scene.thea_channelShadow = bpy.props.BoolProperty(   
                name="Shadow",
                description="Shadow",
                default= False)

Scene.thea_channelMask = bpy.props.BoolProperty(   
                name="Mask",
                description="Mask",
                default= False)

Scene.thea_channelRawDiffuseColor = bpy.props.BoolProperty(   
                name="Raw Diffuse Color",
                description="Raw Diffuse Color",
                default= False)

Scene.thea_channelRawDiffuseLighting = bpy.props.BoolProperty(   
                name="Raw Diffuse Lighting",
                description="Raw Diffuse Lighting",
                default= False)

Scene.thea_channelRawDiffuseGI = bpy.props.BoolProperty(   
                name="Raw Diffuse GI",
                description="Raw Diffuse GI",
                default= False)

Scene.thea_channelSelfIllumination = bpy.props.BoolProperty(   
                name="Self Illumination",
                description="Self Illumination",
                default= False)

Scene.thea_channelDirect = bpy.props.BoolProperty(   
                name="Direct",
                description="Direct",
                default= False)

Scene.thea_channelAO = bpy.props.BoolProperty(   
                name="AO",
                description="AO",
                default= False)

Scene.thea_channelGI = bpy.props.BoolProperty(   
                name="GI",
                description="GI",
                default= False)


Scene.thea_channelSSS = bpy.props.BoolProperty(   
                name="SSS",
                description="SSS",
                default= False)

Scene.thea_channelReflection = bpy.props.BoolProperty(   
                name="Reflection",
                description="Reflection",
                default= False)

Scene.thea_channelRefraction = bpy.props.BoolProperty(   
                name="Refraction",
                description="Refraction",
                default= False)

Scene.thea_channelTransparent = bpy.props.BoolProperty(   
                name="Transparent",
                description="Transparent",
                default= False)

Scene.thea_channelIrradiance = bpy.props.BoolProperty(   
                name="Irradiance",
                description="Irradiance",
                default= False)

Scene.thea_channelSeparatePassesPerLight = bpy.props.BoolProperty(   
                name="Separate Passes Per Light",
                description="Separate Passes Per Light",
                default= False)

Scene.thea_channelInvertMask = bpy.props.BoolProperty(   
                name="Invert Mask Channel",
                description="Invert Mask Channel",
                default= False)

Scene.thea_startTheaAfterExport = bpy.props.BoolProperty(   
                name="start Thea after export",
                description="start Thea after export",
                default= False)

Scene.thea_DispBloom = bpy.props.BoolProperty(   
                name="Glare",
                description="Glare",
                default= False,
                update=displayUpdated)

Scene.thea_DispBloomWeight = bpy.props.FloatProperty(  
                min=0, 
                max=300,
                precision=1,
                default=20,
                name="Glare weight",
                description="Glare weight",
                subtype='PERCENTAGE',
                update=displayUpdated)

Scene.thea_DispGlareRadius = bpy.props.FloatProperty(  
                min=0, 
                max=300,
                precision=1,
                default=4,
                name="Radius",
                description="Glare radius",
                update=displayUpdated)
                
Scene.thea_DispVignetting = bpy.props.BoolProperty(   
                name="Vignetting",
                description="Vignetting",
                default= False,
                update=displayUpdated)                

Scene.thea_DispVignettingWeight = bpy.props.FloatProperty(  
                min=0, 
                max=100,
                precision=1,
                default=20,
                name="Vignetting",
                description="Vignetting weight",
                subtype='PERCENTAGE',
                update=displayUpdated)

Scene.thea_DispSharpness = bpy.props.BoolProperty(   
                name="Sharpness",
                description="Sharpness",
                default= True,
                update=displayUpdated)                

Scene.thea_DispSharpnessWeight = bpy.props.FloatProperty(  
                min=0, 
                max=100,
                precision=1,
                default=50,
                name="Sharpness",
                description="Sharpness weight",
                subtype='PERCENTAGE',
                update=displayUpdated)                

Scene.thea_DispBurn = bpy.props.BoolProperty(   
                name="Burn",
                description="Burn",
                default= True,
                update=displayUpdated)                

Scene.thea_DispBurnWeight = bpy.props.FloatProperty(  
                min=0, 
                max=100,
                precision=1,
                default=10,
                name="Burn",
                description="Burn",
                subtype='PERCENTAGE',
                update=displayUpdated)

Scene.thea_DispCRFMenu = bpy.props.EnumProperty(   
                items=getTheaCRFMenuItems(),
                name="CRF",
                description="CRF",
                default="0",
                update=displayUpdated)                

Scene.thea_DispMinZ = bpy.props.FloatProperty(  
                min=0.0, 
                max=100000000,
                precision=3,
                default=0.000,
                name="Min Z (m)",
                description="Min Z (m)",
                update=displayUpdated)

Scene.thea_DispMaxZ = bpy.props.FloatProperty(  
                min=0.0, 
                max=100000000,
                precision=3,
                default=10.000,
                name="Max Z (m)",
                description="Max Z (m)",
                update=displayUpdated)                

Scene.thea_mergeFilePath = bpy.props.StringProperty(   
                name="Merge scene",
                description="Merge Thea scene",
                default= "")                
                
Scene.thea_SceneMerModels = bpy.props.EnumProperty(   
                items=(('0', 'Keep Current - Throw Away New', ''),('1', 'Replace With New - Throw Away Current', ''), ('2', 'Merge - Add New To Current',''),('3', 'Merge - Name Collision: Keep', ''), ('4', 'Merge - Name Collision: Replace', ''), ('5', 'Merge - Name Collision: Replace Mesh', ''), ('6', 'Merge - Name Collision: Replace Material', '')),
                name="Models",
                description="How to merge models",
                default="0")

Scene.thea_SceneMerLights = bpy.props.EnumProperty(   
                items=(('0', 'Keep Current - Throw Away New', ''),('1', 'Replace With New - Throw Away Current', ''), ('2', 'Merge - Add New To Current',''),('3', 'Merge - Name Collision: Keep', ''), ('4', 'Merge - Name Collision: Replace', ''), ('5', 'Merge - Name Collision: Replace Possition', ''), ('6', 'Merge - Name Collision: Replace Emitter', '')),
                name="Lights",
                description="How to merge lights",
                default="0")

Scene.thea_SceneMerCameras = bpy.props.EnumProperty(   
                items=(('0', 'Keep Current - Throw Away New', ''),('1', 'Replace With New - Throw Away Current', ''), ('2', 'Merge - Add New To Current',''),('3', 'Merge - Name Collision: Keep', ''), ('4', 'Merge - Name Collision: Replace', ''), ('5', 'Merge - Name Collision: Replace Mesh', ''), ('6', 'Merge - Name Collision: Replace Lens', '')),
                name="Cameras",
                description="How to merge cameras",
                default="0")

Scene.thea_SceneMerRender = bpy.props.EnumProperty(   
                items=(('0', 'Keep Current - Throw Away New', ''),('1', 'Replace With New - Throw Away Current', '')),
                name="Render",
                description="How to merge render settings",
                default="0") 

Scene.thea_SceneMerMaterials = bpy.props.EnumProperty(   
                items=(('0', 'Keep Current - Throw Away New', ''),('1', 'Merge - Name Collision: Keep', ''), ('2', 'Merge - Name Collision: Replace', '')),
                name="Materials",
                description="How to merge materials",
                default="0")

Scene.thea_SceneMerEnv = bpy.props.EnumProperty(   
                items=(('0', 'Keep Current - Throw Away New', ''),('1', 'Replace With New - Throw Away Current', '')),
                name="Environment",
                description="How to merge environment",
                default="0")

Scene.thea_SceneMerReverseOrder = bpy.props.BoolProperty(   
                name="Load Thea scene first",
                description="Load Thea scene first",
                default= False)                             
                
Scene.thea_AOEnable = bpy.props.BoolProperty(   
                name="Enable",
                description="Enable AO",
                default= False)                          
                
Scene.thea_AOLowColor = bpy.props.FloatVectorProperty(
                min=0, max=1,  
                name="Low Color", 
                description="AO Low Color", 
                subtype="COLOR")
                
Scene.thea_AOHighColor = bpy.props.FloatVectorProperty(
                min=0, max=1,  
                name="High Color", 
                description="AO Low Color", 
                subtype="COLOR",
                default=(1.0,1.0,1.0))

Scene.thea_AOMultiply = bpy.props.BoolProperty(   
                name="Multiply",
                description="Multiply",
                default= False)                 

Scene.thea_AOClamp = bpy.props.BoolProperty(   
                name="Clamp",
                description="Clamp",
                default= False)

Scene.thea_AOSamples = bpy.props.IntProperty(   
                min=1, 
                max=1000,
                default=100,
                name="Samples",
                description="Samples")                
                
Scene.thea_AODistance = bpy.props.FloatProperty(  
                min=0, 
                max=1000,
                precision=2,
                default=10,
                name="Distance",
                description="Distance")  

Scene.thea_AOIntensity = bpy.props.FloatProperty(  
                min=0, 
                max=1000,
                precision=2,
                default=10,
                name="Intensity",
                description="Intensity")

def materialLutUpdated(self, context):
    lut = getLUTarray()
    mat = context.material
    print("mat.get('thea_LUT')", mat.get('thea_LUT'))
    if mat.use_cubic: 
        mat.use_cubic = False
    else:
        mat.use_cubic = True
    if thea_globals.getNameBasedLUT() and mat.get('thea_LUT') > 0:
       mat.name = lut[mat.get('thea_LUT')]
    updateActiveMaterialColor()

       

def materialUpdated(self, context):
    
    mat = context.material
    print("updated")
    if thea_globals.materialUpdatesNumber<10:
        thea_globals.materialUpdatesNumber += 2      
    
    mat.thea_MaterialLayoutVersion = 2.0;
    # change some unused value to force material preview render
    if mat.use_object_color:
        mat.use_object_color = False
    else:
        mat.use_object_color = True
    

def diffuseColorUpdated(self, context):
    mat = context.material
    mat.diffuse_color = getattr(mat, "thea_BasicDiffuseCol")    
    materialUpdated(context)


def materialFilenameUpdated(self, context, origin=""):
    print("materialFilenameUpdated: ",self, context, origin, context.material.get(origin))
    mat = context.material
    imgName = context.material.get(origin)    
    texName = mat.name+"_"+origin
    exists = False
    try:
        if mat.texture_slots[texName]:
            exists = True
            slot = mat.texture_slots[texName]
            tex = slot.texture                       
    except:
        pass
            
    if exists:
        try:
            if imgName:
                img = bpy.data.images.load(imgName)
                tex.image = img
            else:
                print("removing texture: ", slot, tex)
                mat.texture_slots[texName].texture = None
        except:
            pass
    else:
        img = bpy.data.images.load(imgName)
        tex = bpy.data.textures.new(name=texName, type='IMAGE') 
        tex.image = img
        tex.name = texName
        slot = mat.texture_slots.add()
        slot.texture = tex
        slot.texture_coords='UV'
        if 'Diffuse' in tex.name:
            slot.use_map_color_diffuse=True
        if 'Reflectance' in tex.name:
            slot.use_map_color_spec=True
            slot.use_map_color_diffuse=False
        if 'Translucent' in tex.name:
            slot.use_map_translucency=True
            slot.use_map_color_diffuse=False
        if 'Bump' in tex.name:
            slot.use_map_normal=True
            slot.use_map_color_diffuse=False
            slot.use_map_color_diffuse=False
        if 'Weight' in tex.name:
            slot.use_stencil=True
            slot.use_map_color_diffuse=False
        if 'Roughness' in tex.name:
            slot.use_map_hardness=True
            slot.use_map_color_diffuse=False
        if 'Sigma' in tex.name:
            slot.texture.thea_StructureSigmaTex=True
            slot.use_map_color_diffuse=False
        if 'Anisotropy' in tex.name:
            slot.texture.thea_StructureAnisotropyTex=True
            slot.use_map_color_diffuse=False
        if 'Rotation' in tex.name:
            slot.texture.thea_StructureRotationTex=True
            slot.use_map_color_diffuse=False
        if 'Absorption' in tex.name:
            slot.texture.thea_MediumAbsorptionTex=True
            slot.texture.thea_Medium=True
            slot.use_map_color_diffuse=False
        if 'Scatter' in tex.name:
            slot.texture.thea_MediumScatterTex=True
            slot.texture.thea_Medium=True
            slot.use_map_color_diffuse=False
        if 'AbsorptionDensity' in tex.name:
            slot.texture.thea_MediumAbsorptionDensityTex=True
            slot.texture.thea_Medium=True
            slot.use_map_color_diffuse=False
        if 'ScatterDensity' in tex.name:
            slot.texture.thea_MediumScatterDensityTex=True
            slot.texture.thea_Medium=True
            slot.use_map_color_diffuse=False
        if 'Transmittance' in tex.name:
            slot.use_map_alpha=True
            slot.use_map_color_diffuse=False
        if 'Thickness' in tex.name:
            slot.texture.thea_CoatingThicknessTex=True
            slot.use_map_color_diffuse=False
        if 'Basic' in tex.name:
            slot.texture.thea_Basic=True
        if 'Basic2' in tex.name:
            slot.texture.thea_Basic2=True
        if 'Glossy' in tex.name:
            slot.texture.thea_Glossy=True
        if 'Glossy2' in tex.name:
            slot.texture.thea_Glossy=True
        if 'SSS' in tex.name:
            slot.texture.thea_SSS=True
        if 'ThinFilm' in tex.name:
            slot.texture.thea_ThinFilm=True
        if 'Coating' in tex.name:
            slot.texture.thea_Coating=True
        if 'Clipping' in tex.name:
            slot.texture.thea_Clipping=True
            slot.use_map_alpha=True
        if 'Displacement' in tex.name:
            #slot.texture.thea_Displacement=True
            slot.use_map_displacement=True
        if 'Emittance' in tex.name:
            slot.texture.thea_Emittance=True
    
    if thea_globals.materialUpdatesNumber<10:
        thea_globals.materialUpdatesNumber += 1
    mat.thea_MaterialLayoutVersion = 2.0;
    materialUpdated(context)
            
Mat.thea_OldLayout = bpy.props.BoolProperty(   
                name="Old layout",
                description="Show old material layout",
                default= False)

def updateUnbiasedPreview(self, context):
    mat = context.material
    thea_globals.unbiasedPreview = getattr(mat, "thea_EnableUnbiasedPreview")

Mat.thea_EnableUnbiasedPreview = bpy.props.BoolProperty(   
                name="Use unbiased engine for rendering preview",
                description="Use unbiased engine for rendering preview",
                default= False,
                update=updateUnbiasedPreview)

def updatePreviewScene(self, context):
    mat = context.material
    thea_globals.previewScene = getTheaPreviewScenesMenuItems()[int(getattr(mat, "thea_PreviewScenesMenu"))][1]+".scn.thea"
    print("thea_globals.previewScene: ", thea_globals.previewScene)

 
Mat.thea_PreviewScenesMenu = bpy.props.EnumProperty(   
                items=getTheaPreviewScenesMenuItems(),
                name="Thea preview scene",
                description="Thea preview scene",
                update=updatePreviewScene)

Mat.thea_MaterialLayoutVersion = bpy.props.FloatProperty(  
                min=0, 
                max=100,
                default=0,
                name="Material Layout Version",
                description="Material Layout Version")   
                
Mat.thea_LUT = bpy.props.EnumProperty(   
                items=getLUT(),
                name="Thea Material",
                description="Material from LUT file",
                default="0",
                update=materialLutUpdated)                      

Mat.thea_ShadowCatcher = bpy.props.BoolProperty(   
                name="Shadow Catcher",
                description="Shadow Catcher",
                default= False,
                update=materialUpdated)
                
Mat.thea_Basic = bpy.props.BoolProperty(   
                name="Basic",
                description="Enable Basic material component",
                default= True,
                update=materialUpdated)                                          

Mat.thea_BasicOrder = bpy.props.IntProperty(  
                min=0, 
                max=10,
                default=0,
                name="Order",
                description="Order",
                update=materialUpdated)   

Mat.thea_BasicWeight = bpy.props.FloatProperty(  
                min=0, 
                max=100,
                precision=2,
                default=100.0,
                name="Layer Weight",
                description="Layer Weight",
                update=materialUpdated)     

Mat.thea_BasicWeightFilename = bpy.props.StringProperty(
                  name = "Weight texture",
                  default = "",
                  description = "File path",
                  subtype = 'FILE_PATH',
                  update=lambda a,b: materialFilenameUpdated(a,b,"thea_BasicWeightFilename")
                  )             
                
Mat.thea_BasicDiffuseCol = bpy.props.FloatVectorProperty(
                min=0, max=1,  
                default=(0.5, 0.5, 0.5),
                name="Diffuse Color", 
                description="Diffuse Color", 
                subtype="COLOR",
                update=diffuseColorUpdated)

    
Mat.thea_BasicDiffuseFilename = bpy.props.StringProperty(
                  name = "Diffuse Color texture",
                  default = "",
                  description = "File path",
                  subtype = 'FILE_PATH',
                  update=lambda a,b: materialFilenameUpdated(a,b,"thea_BasicDiffuseFilename")
                  )

Mat.thea_BasicReflectanceCol = bpy.props.FloatVectorProperty(
                min=0, max=1,  
                name="Reflectance Color",
                default=(0, 0, 0), 
                description="Reflectance Color", 
                subtype="COLOR",
                update=materialUpdated)        

Mat.thea_BasicReflectanceFilename = bpy.props.StringProperty(
                  name = "Reflectance Color texture",
                  default = "",
                  description = "File path",
                  subtype = 'FILE_PATH',
                  update=lambda a,b: materialFilenameUpdated(a,b,"thea_BasicReflectanceFilename")
                  )
                
Mat.thea_BasicTranslucentCol = bpy.props.FloatVectorProperty(
                min=0, max=1,  
                name="Translucent Color", 
                description="Translucent Color", 
                subtype="COLOR",
                update=materialUpdated)                        

Mat.thea_BasicTranslucentFilename = bpy.props.StringProperty(
                  name = "Translucent Color texture",
                  default = "",
                  description = "File path",
                  subtype = 'FILE_PATH',
                  update=lambda a,b: materialFilenameUpdated(a,b,"thea_BasicTranslucentFilename")
                  )

Mat.thea_BasicAbsorptionCol = bpy.props.FloatVectorProperty(
                min=0, max=1,  
                name="Absorption Color", 
                description="Absorption Color", 
                subtype="COLOR",
                update=materialUpdated)                        

Mat.thea_BasicAbsorption = bpy.props.FloatProperty(  
                min=0, 
                max=1000,
                precision=2,
                default=0.0,
                name="Absorption",
                description="Absorption",
                update=materialUpdated)                  

Mat.thea_BasicIOR = bpy.props.FloatProperty(  
                min=0, 
                max=1000,
                precision=3,
                default=1.5,
                name="Index of Refraction (n)",
                description="Index of Refraction (n)",
                update=materialUpdated)

Mat.thea_BasicEC = bpy.props.FloatProperty(  
                min=0, 
                max=1000,
                precision=3,
                default=0.0,
                name="Extinction Coefficient (k)",
                description="Extinction Coefficient (k)",
                update=materialUpdated)                

Mat.thea_BasicTrace = bpy.props.BoolProperty(   
                name="Trace Reflections",
                description="Trace Reflections",
                default= True,
                update=materialUpdated)                                          
                
Mat.thea_BasicStructureSigma = bpy.props.FloatProperty(  
                min=0, 
                max=100,
                precision=2,
                default=0.0,
                name="Sigma (%)",
                subtype='PERCENTAGE',
                description="Sigma (%)",
                update=materialUpdated)
                
Mat.thea_BasicStructureRoughness = bpy.props.FloatProperty(  
                min=0, 
                max=100,
                precision=2,
                default=10.0,
                name="Roughness (%)",
                description="Roughness (%)",
                subtype='PERCENTAGE',
                update=materialUpdated)                
                
Mat.thea_BasicStructureAnisotropy = bpy.props.FloatProperty(  
                min=0, 
                max=100,
                precision=2,
                default=0.0,
                name="Anisotropy (%)",
                description="Anisotropy (%)",
                subtype='PERCENTAGE',
                update=materialUpdated)
                
Mat.thea_BasicStructureRotation = bpy.props.FloatProperty(  
                min=0, 
                max=100,
                precision=2,
                default=0.0,
                name="Rotation (deg)",
                description="Rotation (deg)",
                update=materialUpdated)                
                
Mat.thea_BasicStructureBump = bpy.props.FloatProperty(  
                min=0, 
                max=100,
                precision=2,
                default=100.0,
                name="Bump (%)",
                description="Bump (%)",
                subtype='PERCENTAGE',
                update=materialUpdated)        

Mat.thea_BasicBumpFilename = bpy.props.StringProperty(
                  name = "Bumpmap texture",
                  default = "",
                  description = "File path",
                  subtype = 'FILE_PATH',
                  update=lambda a,b: materialFilenameUpdated(a,b,"thea_BasicBumpFilename")
                  )

Mat.thea_BasicSigmaFilename = bpy.props.StringProperty(
                  name = "Sigma texture",
                  default = "",
                  description = "File path",
                  subtype = 'FILE_PATH',
                  update=lambda a,b: materialFilenameUpdated(a,b,"thea_BasicSigmaFilename")
                  )        

Mat.thea_BasicRoughnessFilename = bpy.props.StringProperty(
                  name = "Roughness texture",
                  default = "",
                  description = "File path",
                  subtype = 'FILE_PATH',
                  update=lambda a,b: materialFilenameUpdated(a,b,"thea_BasicRoughnessFilename")
                  )

Mat.thea_BasicAnisotropyFilename = bpy.props.StringProperty(
                  name = "Anisotropy texture",
                  default = "",
                  description = "File path",
                  subtype = 'FILE_PATH',
                  update=lambda a,b: materialFilenameUpdated(a,b,"thea_BasicAnisotropyFilename")
                  )

Mat.thea_BasicRotationFilename = bpy.props.StringProperty(
                  name = "Rotation texture",
                  default = "",
                  description = "File path",
                  subtype = 'FILE_PATH',
                  update=lambda a,b: materialFilenameUpdated(a,b,"thea_BasicRotationFilename")
                  )

Mat.thea_BasicStructureNormal = bpy.props.BoolProperty(   
                name="Normal mapping",
                description="Enable normal mapping",
                default= False,
                update=materialUpdated)                                                          
                
Mat.thea_Basic2 = bpy.props.BoolProperty(   
                name="Basic2",
                description="Enable second Basic material component",
                default= False,
                update=materialUpdated)                                          

Mat.thea_Basic2Order = bpy.props.IntProperty(  
                min=0, 
                max=10,
                default=0,
                name="Order",
                description="Order",
                update=materialUpdated)
                
Mat.thea_Basic2Weight = bpy.props.FloatProperty(  
                min=0, 
                max=100,
                precision=2,
                default=100.0,
                name="Layer Weight",
                description="Layer Weight",
                update=materialUpdated)

Mat.thea_Basic2WeightFilename = bpy.props.StringProperty(
                  name = "Weight texture",
                  default = "",
                  description = "File path",
                  subtype = 'FILE_PATH',
                  update=lambda a,b: materialFilenameUpdated(a,b,"thea_Basic2WeightFilename")
                  )                                  
                
Mat.thea_Basic2DiffuseCol = bpy.props.FloatVectorProperty(
                min=0, max=1,  
                name="Diffuse Color",
                default=(0.5, 0.5, 0.5), 
                description="Diffuse Color", 
                subtype="COLOR",
                update=materialUpdated)

Mat.thea_Basic2DiffuseFilename = bpy.props.StringProperty(
                  name = "Diffuse Color texture",
                  default = "",
                  description = "File path",
                  subtype = 'FILE_PATH',
                  update=lambda a,b: materialFilenameUpdated(a,b,"thea_Basic2DiffuseFilename")
                  )

Mat.thea_Basic2ReflectanceCol = bpy.props.FloatVectorProperty(
                min=0, max=1,  
                name="Reflectance Color",
                default=(0, 0, 0), 
                description="Reflectance Color", 
                subtype="COLOR",
                update=materialUpdated)        

Mat.thea_Basic2ReflectanceFilename = bpy.props.StringProperty(
                  name = "Reflectance Color texture",
                  default = "",
                  description = "File path",
                  subtype = 'FILE_PATH',
                  update=lambda a,b: materialFilenameUpdated(a,b,"thea_Basic2ReflectanceFilename")
                  )
                
                
Mat.thea_Basic2TranslucentCol = bpy.props.FloatVectorProperty(
                min=0, max=1,  
                name="Translucent Color", 
                description="Translucent Color", 
                subtype="COLOR",
                update=materialUpdated) 

Mat.thea_Basic2TranslucentFilename = bpy.props.StringProperty(
                  name = "Translucent Color texture",
                  default = "",
                  description = "File path",
                  subtype = 'FILE_PATH',
                  update=lambda a,b: materialFilenameUpdated(a,b,"thea_Basic2TranslucentFilename")
                  )                       

Mat.thea_Basic2AbsorptionCol = bpy.props.FloatVectorProperty(
                min=0, max=1,  
                name="Absorption Color", 
                description="Absorption Color", 
                subtype="COLOR",
                update=materialUpdated)                        

Mat.thea_Basic2Absorption = bpy.props.FloatProperty(  
                min=0, 
                max=1000,
                precision=2,
                default=0.0,
                name="Absorption",
                description="Absorption",
                update=materialUpdated)                  

Mat.thea_Basic2IOR = bpy.props.FloatProperty(  
                min=0, 
                max=1000,
                precision=3,
                default=1.5,
                name="Index of Refraction (n)",
                description="Index of Refraction (n)",
                update=materialUpdated)

Mat.thea_Basic2EC = bpy.props.FloatProperty(  
                min=0, 
                max=1000,
                precision=3,
                default=0.0,
                name="Extinction Coefficient (k)",
                description="Extinction Coefficient (k)",
                update=materialUpdated)                

Mat.thea_Basic2Trace = bpy.props.BoolProperty(   
                name="Trace Reflections",
                description="Trace Reflections",
                default= True,
                update=materialUpdated)                                          
                
Mat.thea_Basic2StructureSigma = bpy.props.FloatProperty(  
                min=0, 
                max=100,
                precision=2,
                default=0.0,
                name="Sigma (%)",
                subtype='PERCENTAGE',
                description="Sigma (%)",
                update=materialUpdated)
                
Mat.thea_Basic2StructureRoughness = bpy.props.FloatProperty(  
                min=0, 
                max=100,
                precision=2,
                default=10.0,
                name="Roughness (%)",
                subtype='PERCENTAGE',
                description="Roughness (%)",
                update=materialUpdated)                
                
Mat.thea_Basic2StructureAnisotropy = bpy.props.FloatProperty(  
                min=0, 
                max=100,
                precision=2,
                default=0.0,
                name="Anisotropy (%)",
                subtype='PERCENTAGE',
                description="Anisotropy (%)",
                update=materialUpdated)
                
Mat.thea_Basic2StructureRotation = bpy.props.FloatProperty(  
                min=0, 
                max=100,
                precision=2,
                default=0.0,
                name="Rotation (deg)",
                description="Rotation (deg)")                
                
Mat.thea_Basic2StructureBump = bpy.props.FloatProperty(  
                min=0, 
                max=100,
                precision=2,
                default=100.0,
                name="Bump (%)",
                subtype='PERCENTAGE',
                description="Bump (%)",
                update=materialUpdated)       

Mat.thea_Basic2BumpFilename = bpy.props.StringProperty(
                  name = "Bumpmap texture",
                  default = "",
                  description = "File path",
                  subtype = 'FILE_PATH',
                  update=lambda a,b: materialFilenameUpdated(a,b,"thea_Basic2BumpFilename")
                  )    

Mat.thea_Basic2SigmaFilename = bpy.props.StringProperty(
                  name = "Sigma texture",
                  default = "",
                  description = "File path",
                  subtype = 'FILE_PATH',
                  update=lambda a,b: materialFilenameUpdated(a,b,"thea_Basic2SigmaFilename")
                  )        

Mat.thea_Basic2RoughnessFilename = bpy.props.StringProperty(
                  name = "Roughness texture",
                  default = "",
                  description = "File path",
                  subtype = 'FILE_PATH',
                  update=lambda a,b: materialFilenameUpdated(a,b,"thea_Basic2RoughnessFilename")
                  )

Mat.thea_Basic2AnisotropyFilename = bpy.props.StringProperty(
                  name = "Anisotropy texture",
                  default = "",
                  description = "File path",
                  subtype = 'FILE_PATH',
                  update=lambda a,b: materialFilenameUpdated(a,b,"thea_Basic2AnisotropyFilename")
                  )

Mat.thea_Basic2RotationFilename = bpy.props.StringProperty(
                  name = "Rotation texture",
                  default = "",
                  description = "File path",
                  subtype = 'FILE_PATH',
                  update=lambda a,b: materialFilenameUpdated(a,b,"thea_Basic2RotationFilename")
                  )      

Mat.thea_Basic2StructureNormal = bpy.props.BoolProperty(   
                name="Normal mapping",
                description="Enable normal mapping",
                default= False,
                update=materialUpdated)                                                                          
                
Mat.thea_Glossy = bpy.props.BoolProperty(   
                name="Glossy",
                description="Enable Glossy material component",
                default= False,
                update=materialUpdated)                                          

Mat.thea_GlossyOrder = bpy.props.IntProperty(  
                min=0, 
                max=10,
                default=0,
                name="Order",
                description="Order",
                update=materialUpdated)

Mat.thea_GlossyWeight = bpy.props.FloatProperty(  
                min=0, 
                max=100,
                precision=2,
                default=100.0,
                name="Layer Weight",
                description="Layer Weight",
                update=materialUpdated) 

Mat.thea_GlossyWeightFilename = bpy.props.StringProperty(
                  name = "Weight texture",
                  default = "",
                  description = "File path",
                  subtype = 'FILE_PATH',
                  update=lambda a,b: materialFilenameUpdated(a,b,"thea_GlossyWeightFilename")
                  )                 
                
Mat.thea_GlossyReflectanceCol = bpy.props.FloatVectorProperty(
                min=0, max=1,  
                default=(1, 1, 1),
                name="Reflectance Color", 
                description="Reflectance Color", 
                subtype="COLOR",
                update=materialUpdated)   

Mat.thea_GlossyReflectanceFilename = bpy.props.StringProperty(
                  name = "Reflectance texture",
                  default = "",
                  description = "File path",
                  subtype = 'FILE_PATH',
                  update=lambda a,b: materialFilenameUpdated(a,b,"thea_GlossyReflectanceFilename")
                  )     
                
Mat.thea_GlossyTransmittanceCol = bpy.props.FloatVectorProperty(
                min=0, max=1,  
                name="Transmittance Color", 
                description="Transmittance Color", 
                subtype="COLOR",
                update=materialUpdated)                        

Mat.thea_GlossyTransmittanceFilename = bpy.props.StringProperty(
                  name = "Transmittance texture",
                  default = "",
                  description = "File path",
                  subtype = 'FILE_PATH',
                  update=lambda a,b: materialFilenameUpdated(a,b,"thea_GlossyTransmittanceFilename")
                  )     

Mat.thea_GlossyAbsorptionCol = bpy.props.FloatVectorProperty(
                min=0, max=1,  
                name="Absorption Color", 
                description="Absorption Color", 
                subtype="COLOR",
                update=materialUpdated)                        

Mat.thea_GlossyAbsorption = bpy.props.FloatProperty(  
                min=0, 
                max=1000,
                precision=2,
                default=0.0,
                name="Absorption",
                description="Absorption",
                update=materialUpdated)                  

Mat.thea_GlossyIOR = bpy.props.FloatProperty(  
                min=0, 
                max=1000,
                precision=3,
                default=1.5,
                name="Index of Refraction (n)",
                description="Index of Refraction (n)",
                update=materialUpdated)

Mat.thea_GlossyEC = bpy.props.FloatProperty(  
                min=0, 
                max=1000,
                precision=3,
                default=0.0,
                name="Extinction Coefficient (k)",
                description="Extinction Coefficient (k)",
                update=materialUpdated)                

Mat.thea_GlossyAbbeNumberEnable = bpy.props.BoolProperty(   
                name="Abbe",
                description="Abbe",
                default= False,
                update=materialUpdated)
                
Mat.thea_GlossyAbbeNumber = bpy.props.FloatProperty(  
                min=0, 
                max=200,
                precision=3,
                default=50.0,
                name="Number",
                description="Number",
                update=materialUpdated)      

Mat.thea_GlossyIORFileEnable = bpy.props.BoolProperty(   
                name="IOR file",
                description="IOR file",
                default= False,
                update=materialUpdated)

Mat.thea_GlossyIORMenu = bpy.props.EnumProperty(   
                items=getTheaIORMenuItems(),
                name="File",
                description="File",
                update=materialUpdated)                          

Mat.thea_GlossyTraceReflections = bpy.props.BoolProperty(   
                name="Trace Reflections",
                description="Trace Reflections",
                default= True,
                update=materialUpdated)                                          

Mat.thea_GlossyTraceRefractions = bpy.props.BoolProperty(   
                name="Trace Refractions",
                description="Trace Refractions",
                default= True,
                update=materialUpdated)                                                          

Mat.thea_GlossyStructureRoughness = bpy.props.FloatProperty(  
                min=0, 
                max=100,
                precision=2,
                default=0.0,
                name="Roughness (%)",
                subtype='PERCENTAGE',
                description="Roughness (%)",
                update=materialUpdated)

Mat.thea_GlossyStructureRoughTrEn = bpy.props.BoolProperty(   
                name="Roughness Tr (%)",
                subtype='PERCENTAGE',
                description="Roughness Tr (%)",
                default= False,
                update=materialUpdated)                                                          
                
                
Mat.thea_GlossyStructureRoughnessTr = bpy.props.FloatProperty(  
                min=0, 
                max=100,
                precision=2,
                default=10.0,
                name="(%)",
                subtype='PERCENTAGE',
                description="(%)",
                update=materialUpdated)                
                
Mat.thea_GlossyStructureAnisotropy = bpy.props.FloatProperty(  
                min=0, 
                max=100,
                precision=2,
                default=0.0,
                name="Anisotropy (%)",
                description="Anisotropy (%)",
                update=materialUpdated)
                
Mat.thea_GlossyStructureRotation = bpy.props.FloatProperty(  
                min=0, 
                max=100,
                precision=2,
                default=0.0,
                name="Rotation (deg)",
                description="Rotation (deg)",
                update=materialUpdated)                
                
Mat.thea_GlossyStructureBump = bpy.props.FloatProperty(  
                min=0, 
                max=100,
                precision=2,
                default=100.0,
                name="Bump (%)",
                description="Bump (%)",
                subtype='PERCENTAGE',
                update=materialUpdated)  

Mat.thea_GlossySigmaFilename = bpy.props.StringProperty(
                  name = "Sigma texture",
                  default = "",
                  description = "File path",
                  subtype = 'FILE_PATH',
                  update=lambda a,b: materialFilenameUpdated(a,b,"thea_GlossySigmaFilename")
                  )        

Mat.thea_GlossyRoughnessFilename = bpy.props.StringProperty(
                  name = "Roughness texture",
                  default = "",
                  description = "File path",
                  subtype = 'FILE_PATH',
                  update=lambda a,b: materialFilenameUpdated(a,b,"thea_GlossyRoughnessFilename")
                  )

Mat.thea_GlossyAnisotropyFilename = bpy.props.StringProperty(
                  name = "Anisotropy texture",
                  default = "",
                  description = "File path",
                  subtype = 'FILE_PATH',
                  update=lambda a,b: materialFilenameUpdated(a,b,"thea_GlossyAnisotropyFilename")
                  )

Mat.thea_GlossyRotationFilename = bpy.props.StringProperty(
                  name = "Rotation texture",
                  default = "",
                  description = "File path",
                  subtype = 'FILE_PATH',
                  update=lambda a,b: materialFilenameUpdated(a,b,"thea_GlossyRotationFilename")
                  )              

Mat.thea_GlossyBumpFilename = bpy.props.StringProperty(
                  name = "Bump texture",
                  default = "",
                  description = "File path",
                  subtype = 'FILE_PATH',
                  update=lambda a,b: materialFilenameUpdated(a,b,"thea_GlossyBumpFilename")
                  )        

Mat.thea_GlossyStructureNormal = bpy.props.BoolProperty(   
                name="Normal mapping",
                description="Enable normal mapping",
                default= False,
                update=materialUpdated)                                                                          

Mat.thea_Glossy2 = bpy.props.BoolProperty(   
                name="Glossy2",
                description="Enable Glossy2 material component",
                default= False,
                update=materialUpdated)                                          

Mat.thea_Glossy2Order = bpy.props.IntProperty(  
                min=0, 
                max=10,
                default=0,
                name="Order",
                description="Order",
                update=materialUpdated)

Mat.thea_Glossy2Weight = bpy.props.FloatProperty(  
                min=0, 
                max=100,
                precision=2,
                default=100.0,
                name="Layer Weight",
                description="Layer Weight",
                update=materialUpdated) 

Mat.thea_Glossy2WeightFilename = bpy.props.StringProperty(
                  name = "Weight texture",
                  default = "",
                  description = "File path",
                  subtype = 'FILE_PATH',
                  update=lambda a,b: materialFilenameUpdated(a,b,"thea_Glossy2WeightFilename")
                  )                 
                
Mat.thea_Glossy2ReflectanceCol = bpy.props.FloatVectorProperty(
                min=0, max=1,  
                default=(1, 1, 1),
                name="Reflectance Color", 
                description="Reflectance Color", 
                subtype="COLOR",
                update=materialUpdated)   

Mat.thea_Glossy2ReflectanceFilename = bpy.props.StringProperty(
                  name = "Reflectance texture",
                  default = "",
                  description = "File path",
                  subtype = 'FILE_PATH',
                  update=lambda a,b: materialFilenameUpdated(a,b,"thea_Glossy2ReflectanceFilename")
                  )     
                
Mat.thea_Glossy2TransmittanceCol = bpy.props.FloatVectorProperty(
                min=0, max=1,  
                name="Transmittance Color", 
                description="Transmittance Color", 
                subtype="COLOR",
                update=materialUpdated)                        

Mat.thea_Glossy2TransmittanceFilename = bpy.props.StringProperty(
                  name = "Transmittance texture",
                  default = "",
                  description = "File path",
                  subtype = 'FILE_PATH',
                  update=lambda a,b: materialFilenameUpdated(a,b,"thea_Glossy2TransmittanceFilename")
                  )     

Mat.thea_Glossy2AbsorptionCol = bpy.props.FloatVectorProperty(
                min=0, max=1,  
                name="Absorption Color", 
                description="Absorption Color", 
                subtype="COLOR",
                update=materialUpdated)                        

Mat.thea_Glossy2Absorption = bpy.props.FloatProperty(  
                min=0, 
                max=1000,
                precision=2,
                default=0.0,
                name="Absorption",
                description="Absorption",
                update=materialUpdated)                  

Mat.thea_Glossy2IOR = bpy.props.FloatProperty(  
                min=0, 
                max=1000,
                precision=3,
                default=1.5,
                name="Index of Refraction (n)",
                description="Index of Refraction (n)",
                update=materialUpdated)

Mat.thea_Glossy2EC = bpy.props.FloatProperty(  
                min=0, 
                max=1000,
                precision=3,
                default=0.0,
                name="Extinction Coefficient (k)",
                description="Extinction Coefficient (k)",
                update=materialUpdated)                

Mat.thea_Glossy2AbbeNumberEnable = bpy.props.BoolProperty(   
                name="Abbe",
                description="Abbe",
                default= False,
                update=materialUpdated)
                
Mat.thea_Glossy2AbbeNumber = bpy.props.FloatProperty(  
                min=0, 
                max=200,
                precision=3,
                default=50.0,
                name="Number",
                description="Number",
                update=materialUpdated)

Mat.thea_Glossy2IORFileEnable = bpy.props.BoolProperty(   
                name="IOR file",
                description="IOR file",
                default= False,
                update=materialUpdated)

Mat.thea_Glossy2IORMenu = bpy.props.EnumProperty(   
                items=getTheaIORMenuItems(),
                name="File",
                description="File",
                update=materialUpdated)                                

Mat.thea_Glossy2TraceReflections = bpy.props.BoolProperty(   
                name="Trace Reflections",
                description="Trace Reflections",
                default= True,
                update=materialUpdated)                                          

Mat.thea_Glossy2TraceRefractions = bpy.props.BoolProperty(   
                name="Trace Refractions",
                description="Trace Refractions",
                default= True,
                update=materialUpdated)                                                          

Mat.thea_Glossy2StructureRoughness = bpy.props.FloatProperty(  
                min=0, 
                max=100,
                precision=2,
                default=0.0,
                name="Roughness (%)",
                description="Roughness (%)",
                subtype='PERCENTAGE',
                update=materialUpdated)

Mat.thea_Glossy2StructureRoughTrEn = bpy.props.BoolProperty(   
                name="Roughness Tr (%)",
                description="Roughness Tr (%)",
                default= False,
                update=materialUpdated)                                                          
                
                
Mat.thea_Glossy2StructureRoughnessTr = bpy.props.FloatProperty(  
                min=0, 
                max=100,
                precision=2,
                default=10.0,
                name="(%)",
                description="(%)",
                subtype='PERCENTAGE',
                update=materialUpdated)                
                
Mat.thea_Glossy2StructureAnisotropy = bpy.props.FloatProperty(  
                min=0, 
                max=100,
                precision=2,
                default=0.0,
                name="Anisotropy (%)",
                description="Anisotropy (%)",
                subtype='PERCENTAGE',
                update=materialUpdated)
                
Mat.thea_Glossy2StructureRotation = bpy.props.FloatProperty(  
                min=0, 
                max=100,
                precision=2,
                default=0.0,
                name="Rotation (deg)",
                description="Rotation (deg)",
                update=materialUpdated)                
                
Mat.thea_Glossy2StructureBump = bpy.props.FloatProperty(  
                min=0, 
                max=100,
                precision=2,
                default=100.0,
                name="Bump (%)",
                description="Bump (%)",
                subtype='PERCENTAGE',
                update=materialUpdated)  

Mat.thea_Glossy2SigmaFilename = bpy.props.StringProperty(
                  name = "Sigma texture",
                  default = "",
                  description = "File path",
                  subtype = 'FILE_PATH',
                  update=lambda a,b: materialFilenameUpdated(a,b,"thea_Glossy2SigmaFilename")
                  )        

Mat.thea_Glossy2RoughnessFilename = bpy.props.StringProperty(
                  name = "Roughness texture",
                  default = "",
                  description = "File path",
                  subtype = 'FILE_PATH',
                  update=lambda a,b: materialFilenameUpdated(a,b,"thea_Glossy2RoughnessFilename")
                  )

Mat.thea_Glossy2AnisotropyFilename = bpy.props.StringProperty(
                  name = "Anisotropy texture",
                  default = "",
                  description = "File path",
                  subtype = 'FILE_PATH',
                  update=lambda a,b: materialFilenameUpdated(a,b,"thea_Glossy2AnisotropyFilename")
                  )

Mat.thea_Glossy2RotationFilename = bpy.props.StringProperty(
                  name = "Rotation texture",
                  default = "",
                  description = "File path",
                  subtype = 'FILE_PATH',
                  update=lambda a,b: materialFilenameUpdated(a,b,"thea_Glossy2RotationFilename")
                  )              

Mat.thea_Glossy2BumpFilename = bpy.props.StringProperty(
                  name = "Bump texture",
                  default = "",
                  description = "File path",
                  subtype = 'FILE_PATH',
                  update=lambda a,b: materialFilenameUpdated(a,b,"thea_Glossy2BumpFilename")
                  )        

Mat.thea_Glossy2StructureNormal = bpy.props.BoolProperty(   
                name="Normal mapping",
                description="Enable normal mapping",
                default= False,
                update=materialUpdated)                                                                          


Mat.thea_Displacement = bpy.props.BoolProperty(   
                name="Displacement",
                description="Displacement",
                default= False,
                update=materialUpdated)

Mat.thea_DisplacementFilename = bpy.props.StringProperty(
                  name = "Displacement texture",
                  default = "",
                  description = "File path",
                  subtype = 'FILE_PATH',
                  update=lambda a,b: materialFilenameUpdated(a,b,"thea_DisplacementFilename")
                  ) 
                
Mat.thea_DispSub = bpy.props.IntProperty(
                min=0, 
                max=100, 
                default=2, 
                name="Subdivision", 
                description="Subdivision",
                update=materialUpdated)
                    
Mat.thea_DispBump = bpy.props.IntProperty(
                min=0, 
                max=100, 
                default=0, 
                name="Micro-Bump", 
                description="Micro-Bump",
                update=materialUpdated)
                
Mat.thea_DisplacementHeight = bpy.props.FloatProperty(  
                min=0, 
                max=10000,
                precision=2,
                default=2,
                name="Height (cm)",
                description="Height (cm)",
                update=materialUpdated)
                
Mat.thea_DisplacementCenter = bpy.props.FloatProperty(  
                min=0, 
                max=10,
                precision=2,
                default=0.5,
                name="Center",
                description="Center",
                update=materialUpdated)                
                
Mat.thea_DisplacementNormalSmooth = bpy.props.BoolProperty(   
                name="Normal Smoothing",
                description="Normal Smoothing",
                default= True,
                update=materialUpdated)

Mat.thea_DisplacementTightBounds = bpy.props.BoolProperty(   
                name="Tight Bounds",
                description="Tight Bounds",
                default= True,
                update=materialUpdated)
                
Mat.thea_Clipping = bpy.props.BoolProperty(   
                name="Clipping",
                description="Clipping",
                default= False,
                update=materialUpdated)

Mat.thea_ClippingFilename = bpy.props.StringProperty(
                  name = "Clipping texture",
                  default = "",
                  description = "File path",
                  subtype = 'FILE_PATH',
                  update=lambda a,b: materialFilenameUpdated(a,b,"thea_ClippingFilename")
                  )                

Mat.thea_ClippingThreshold = bpy.props.FloatProperty(  
                min=0, 
                max=100,
                precision=2,
                default=50,
                name="Threshold (%)",
                description="Threshold (%)",
                subtype='PERCENTAGE',
                update=materialUpdated)                
                
Mat.thea_ClippingSoft = bpy.props.BoolProperty(   
                name="Soft",
                description="Soft",
                default= False,
                update=materialUpdated)

Mat.thea_Emittance = bpy.props.BoolProperty(   
                name="Emittance",
                description="Emittance",
                default= False,
                update=materialUpdated)

Mat.thea_EmittancePower = bpy.props.FloatProperty(  
                min=0, 
                max=1000000000,
                precision=3,
                default=100.0,
                name="Power",
                description="Power",
                update=materialUpdated)         

Mat.thea_EmittanceEfficacy = bpy.props.FloatProperty(  
                min=0, 
                max=1000000000,
                precision=3,
                default=20.0,
                name="Efficacy (lm/W)",
                description="Efficacy (lm/W)",
                update=materialUpdated)                           
                
Mat.thea_EmittanceCol = bpy.props.FloatVectorProperty(
                min=0, max=1,  
                name="Color", 
                default=(1,1,1),
                description="Color", 
                subtype="COLOR",
                update=materialUpdated)       

Mat.thea_EmittanceFilename = bpy.props.StringProperty(
                  name = "Emittance texture",
                  default = "",
                  description = "File path",
                  subtype = 'FILE_PATH',
                  update=lambda a,b: materialFilenameUpdated(a,b,"thea_EmittanceFilename")
                  ) 

Mat.thea_EmittanceIES = bpy.props.BoolProperty(   
                name="IES File",
                description="IES File",
                default= False,
                update=materialUpdated)

Mat.thea_EmittanceIESFilename = bpy.props.StringProperty(
                  name = "IES File",
                  default = "",
                  description = "File path",
                  subtype = 'FILE_PATH',
                  update=lambda a,b: materialFilenameUpdated(a,b,"thea_EmittanceFilename")
                  ) 

Mat.thea_EmittanceIESMultiplier = bpy.props.FloatProperty(  
                min=0, 
                max=1000,
                precision=3,
                default=1,
                name="Multiplier",
                description="Multiplier",
                update=materialUpdated)

Mat.thea_EmittanceUnit = bpy.props.EnumProperty(   
                items=(("Lumens","Lumens","Lumens"),("lm/m2","lm/m2","lm/m2"),("Candelas","Candelas","Candelas"),("cd/m2","cd/m2","cd/m2"),("Watts","Watts","Watts"),("W/m2","W/m2","W/m2"),("W/sr","W/sr","W/sr"),("W/sr/m2","W/sr/m2","W/sr/m2"),("W/nm","W/nm","W/nm"),("W/nm/m2","W/nm/m2","W/nm/m2"),("W/nm/sr","W/nm/sr","W/nm/sr"),("W/nm/sr/m2","W/nm/sr/m2","W/nm/sr/m2")),                
                name="Unit",
                description="Unit",
                default="Watts",
                update=materialUpdated)


Mat.thea_EmittancePassiveEmitter = bpy.props.BoolProperty(   
                name="Passive Emitter",
                description="Passive Emitter",
                default= False,
                update=materialUpdated)

Mat.thea_EmittanceAmbientEmitter = bpy.props.BoolProperty(   
                name="Ambient Emitter",
                description="Ambient Emitter",
                default= False,
                update=materialUpdated)

Mat.thea_EmittanceAmbientLevel = bpy.props.FloatProperty(  
                min=0, 
                max=100,
                precision=3,
                default=100.0,
                name="Level",
                description="Level",
                subtype='PERCENTAGE',
                update=materialUpdated)
                
Mat.thea_extMat = bpy.props.StringProperty( 
                       maxlen=500,
                       default="",
                       name="Thea material file",
                       description="Thea material file",
                       subtype='FILE_PATH',
                update=materialUpdated)                 

Mat.thea_Medium = bpy.props.BoolProperty(   
                name="Medium",
                description="Medium",
                default= False,
                update=materialUpdated)

                
Mat.thea_MediumAbsorptionCol = bpy.props.FloatVectorProperty(
                min=0, max=1,  
                name="Absorption Color", 
                default=(1,1,1),
                description="Color", 
                subtype="COLOR",
                update=materialUpdated)

Tex.thea_MediumAbsorptionTex = bpy.props.BoolProperty(   
                name="Absorption Texture",
                description="Absorption Texture",
                default= False,
                update=materialUpdated)                        

Mat.thea_MediumAbsorptionFilename = bpy.props.StringProperty(
                  name = "Absorption texture",
                  default = "",
                  description = "File path",
                  subtype = 'FILE_PATH',
                  update=lambda a,b: materialFilenameUpdated(a,b,"thea_MediumAbsorptionFilename")
                  ) 

Mat.thea_MediumScatterCol = bpy.props.FloatVectorProperty(
                min=0, max=1,  
                name="Scatter Color", 
                default=(1,1,1),
                description="Color", 
                subtype="COLOR",
                update=materialUpdated)

Tex.thea_MediumScatterTex = bpy.props.BoolProperty(   
                name="Scatter Texture",
                description="Scatter Texture",
                default= False,
                update=materialUpdated)               

Mat.thea_MediumScatterFilename = bpy.props.StringProperty(
                  name = "Scatter texture",
                  default = "",
                  description = "File path",
                  subtype = 'FILE_PATH',
                  update=lambda a,b: materialFilenameUpdated(a,b,"thea_MediumScatterFilename")
                  ) 

Mat.thea_MediumAbsorptionDensity = bpy.props.FloatProperty(  
                min=0, 
                max=1000,
                precision=3,
                default=1,
                name="Absorption Density",
                description="Absorption Density",
                update=materialUpdated)

Tex.thea_MediumAbsorptionDensityTex = bpy.props.BoolProperty(   
                name="Absorption Density Texture",
                description="Absorption Density Texture",
                default= False,
                update=materialUpdated)

Mat.thea_MediumAbsorptionDensityFilename = bpy.props.StringProperty(
                  name = "Absorption Density texture",
                  default = "",
                  description = "File path",
                  subtype = 'FILE_PATH',
                  update=lambda a,b: materialFilenameUpdated(a,b,"thea_MediumAbsorptionDensityFilename")
                  )

Mat.thea_MediumScatterDensity = bpy.props.FloatProperty(  
                min=0, 
                max=1000,
                precision=3,
                default=1,
                name="Scatter Density",
                description="Scatter Density",
                update=materialUpdated)

Tex.thea_MediumScatterDensityTex = bpy.props.BoolProperty(   
                name="Scatter Density Texture",
                description="Scatter Density Texture",
                default= False,
                update=materialUpdated)

Mat.thea_MediumScatterDensityFilename = bpy.props.StringProperty(
                  name = "Scatter Density texture",
                  default = "",
                  description = "File path",
                  subtype = 'FILE_PATH',
                  update=lambda a,b: materialFilenameUpdated(a,b,"thea_MediumScatterDensityFilename")
                  )

Mat.thea_MediumCoefficient = bpy.props.BoolProperty(   
                name="Coefficient",
                description="Coefficient",
                default= False,
                update=materialUpdated)

Mat.thea_MediumMenu = bpy.props.EnumProperty(   
                items=getTheaMediumMenuItems(),
                name="File",
                description="File",
                update=materialUpdated)

Mat.thea_MediumPhaseFunction = bpy.props.EnumProperty(   
                items=(("Isotropic","Isotropic","Isotropic"),("Rayleigh","Rayleigh","Rayleigh"),("Mie Hazy","Mie Hazy","Mie Hazy"), ("Mie Murky", "Mie Murky", "Mie Murky"), ("Mie Retro", "Mie Retro", "Mie Retro"), ("Henyey Greenstein", "Henyey Greenstein", "Henyey Greenstein")),
                name="Phase Function",
                description="Phase Function",
                default="Isotropic",
                update=materialUpdated)

Tex.thea_Basic = bpy.props.BoolProperty(   
                name="Basic",
                description="Use with Basic component",
                default= False)
                
Tex.thea_Basic2 = bpy.props.BoolProperty(   
                name="Basic2",
                description="Use with second Basic component",
                default= False)                
                                
Tex.thea_Glossy = bpy.props.BoolProperty(   
                name="Glossy",
                description="Use with Glossy component",
                default= False)
                
Tex.thea_Coating = bpy.props.BoolProperty(   
                name="Coating",
                description="Use with coating component",
                default= False)                                

Tex.thea_ThinFilm = bpy.props.BoolProperty(   
                name="Thin Film",
                description="Use with ThinFilm component",
                default= False)

Tex.thea_SSS = bpy.props.BoolProperty(   
                name="SSS",
                description="Use with SSS component",
                default= False)
                
Tex.thea_Clipping = bpy.props.BoolProperty(   
                name="Clipping",
                description="Use with Clipping component",
                default= False)

Tex.thea_Medium = bpy.props.BoolProperty(   
                name="Medium",
                description="Use with Medium component",
                default= False)        

Tex.thea_Emittance = bpy.props.BoolProperty(   
                name="Emittance",
                description="Use with Emittance component",
                default= False) 

Tex.thea_TexUVChannel = bpy.props.EnumProperty(   
                items=(('0', '0', '0'),('1', '1', '1'),('2', '2', '2'),('3', '3', '3'),('4', '4', '4'),('5', '5', '5'),('6', '6', '6'),('7', '7', '7')),
                name="UV Channel",
                description="UV Channel",
                default="0")       

Tex.thea_TexInvert = bpy.props.BoolProperty(   
                name="Invert",
                description="Invert",
                default= False)                

Tex.thea_TexGamma = bpy.props.FloatProperty(  
                min=-100, 
                max=100,
                precision=2,
                default=0,
                name="Gamma",
                description="Gamma")

Tex.thea_TexRed = bpy.props.FloatProperty(  
                min=-100, 
                max=100,
                precision=2,
                default=0,
                name="Red",
                description="Red")

Tex.thea_TexGreen = bpy.props.FloatProperty(  
                min=-100, 
                max=100,
                precision=2,
                default=0,
                name="Green",
                description="Green")

Tex.thea_TexBlue = bpy.props.FloatProperty(  
                min=-100, 
                max=100,
                precision=2,
                default=0,
                name="Blue",
                description="Blue")

Tex.thea_TexSaturation = bpy.props.FloatProperty(  
                min=-100, 
                max=100,
                precision=2,
                default=0,
                name="Saturation",
                description="Saturation")

Tex.thea_TexBrightness = bpy.props.FloatProperty(  
                min=-100, 
                max=100,
                precision=2,
                default=0,
                name="Brightness",
                description="Brightness")

Tex.thea_TexContrast = bpy.props.FloatProperty(  
                min=-100, 
                max=100,
                precision=2,
                default=0,
                name="Contrast",
                description="Contrast")

Tex.thea_TexClampMin = bpy.props.FloatProperty(  
                min=0, 
                max=100,
                precision=2,
                default=0,
                name="Clamp Min %",
                description="Clamp Min %")

Tex.thea_TexClampMax = bpy.props.FloatProperty(  
                min=0, 
                max=100,
                precision=2,
                default=100,
                name="Clamp Max %",
                description="Clamp Max %")

Tex.thea_TexRotation = bpy.props.FloatProperty(  
                min=-10000, 
                max=100000,
                precision=2,
                default=0,
                name="Rotation",
                description="Rotation")

Tex.thea_StructureSigmaTex = bpy.props.BoolProperty(   
                name="Sigma Texture",
                description="Sigma Texture",
                default= False)

Tex.thea_StructureAnisotropyTex = bpy.props.BoolProperty(   
                name="Anisotropy Texture",
                description="Anisotropy Texture",
                default= False)                  

Tex.thea_StructureRotationTex = bpy.props.BoolProperty(   
                name="Rotation Texture",
                description="Rotation Texture",
                default= False)

Tex.thea_TransmittanceTex = bpy.props.BoolProperty(   
                name="Transmittance Texture",
                description="Transmittance Texture",
                default= False)                

Mat.thea_Coating = bpy.props.BoolProperty(   
                name="Coating",
                description="Enable Coating material component",
                default= False,
                update=materialUpdated)                                          

Mat.thea_CoatingWeight = bpy.props.FloatProperty(  
                min=0, 
                max=100,
                precision=2,
                default=100.0,
                name="Layer Weight",
                description="Layer Weight",
                update=materialUpdated)      

Mat.thea_CoatingWeightFilename = bpy.props.StringProperty(
                  name = "Weight texture",
                  default = "",
                  description = "File path",
                  subtype = 'FILE_PATH',
                  update=lambda a,b: materialFilenameUpdated(a,b,"thea_CoatingWeightFilename")
                  )            
                
Mat.thea_CoatingReflectanceCol = bpy.props.FloatVectorProperty(
                min=0, max=1,  
                default=(1, 1, 1),
                name="Reflectance Color", 
                description="Reflectance Color", 
                subtype="COLOR",
                update=materialUpdated)

Mat.thea_CoatingReflectanceFilename = bpy.props.StringProperty(
                  name = "Reflectance texture",
                  default = "",
                  description = "File path",
                  subtype = 'FILE_PATH',
                  update=lambda a,b: materialFilenameUpdated(a,b,"thea_CoatingReflectanceFilename")
                  )                  
                
Mat.thea_CoatingIOR = bpy.props.FloatProperty(  
                min=0, 
                max=1000,
                precision=3,
                default=1.5,
                name="Index of Refraction (n)",
                description="Index of Refraction (n)",
                update=materialUpdated)

Mat.thea_CoatingEC = bpy.props.FloatProperty(  
                min=0, 
                max=1000,
                precision=3,
                default=0.0,
                name="Extinction Coefficient (k)",
                description="Extinction Coefficient (k)",
                update=materialUpdated)                

Mat.thea_CoatingThicknessEnable = bpy.props.BoolProperty(   
                name="Thickness (um)",
                description="Thickness (um)",
                default= False,
                update=materialUpdated)
                
Mat.thea_CoatingThickness = bpy.props.FloatProperty(  
                min=0, 
                max=20000,
                precision=3,
                default=100.0,
                name="Number",
                description="Number",
                update=materialUpdated)

Mat.thea_CoatingThicknessFilename = bpy.props.StringProperty(
                  name = "Thickness texture",
                  default = "",
                  description = "File path",
                  subtype = 'FILE_PATH',
                  update=lambda a,b: materialFilenameUpdated(a,b,"thea_CoatingThicknessFilename")
                  )

Tex.thea_CoatingThicknessTex = bpy.props.BoolProperty(   
                name="Thickness Texture",
                description="Thickness Texture",
                default= False,
                update=materialUpdated)                                  

Mat.thea_CoatingTraceReflections = bpy.props.BoolProperty(   
                name="Trace Reflections",
                description="Trace Reflections",
                default= True,
                update=materialUpdated)                                          

Mat.thea_CoatingStructureRoughness = bpy.props.FloatProperty(  
                min=0, 
                max=100,
                precision=2,
                default=0.0,
                name="Roughness (%)",
                description="Roughness (%)",
                subtype='PERCENTAGE',
                update=materialUpdated)                
                                
Mat.thea_CoatingStructureAnisotropy = bpy.props.FloatProperty(  
                min=0, 
                max=100,
                precision=2,
                default=0.0,
                name="Anisotropy (%)",
                description="Anisotropy (%)",
                subtype='PERCENTAGE',
                update=materialUpdated)
                
Mat.thea_CoatingStructureRotation = bpy.props.FloatProperty(  
                min=0, 
                max=100,
                precision=2,
                default=0.0,
                name="Rotation (deg)",
                description="Rotation (deg)")                
                
Mat.thea_CoatingStructureBump = bpy.props.FloatProperty(  
                min=0, 
                max=100,
                precision=2,
                default=100.0,
                name="Bump (%)",
                description="Bump (%)",
                subtype='PERCENTAGE',
                update=materialUpdated)                

Mat.thea_CoatingStructureNormal = bpy.props.BoolProperty(   
                name="Normal mapping",
                description="Enable normal mapping",
                default= False,
                update=materialUpdated)

Mat.thea_CoatingRoughnessFilename = bpy.props.StringProperty(
                  name = "Roughness texture",
                  default = "",
                  description = "File path",
                  subtype = 'FILE_PATH',
                  update=lambda a,b: materialFilenameUpdated(a,b,"thea_CoatingRoughnessFilename")
                  )

Mat.thea_CoatingAnisotropyFilename = bpy.props.StringProperty(
                  name = "Anisotropy texture",
                  default = "",
                  description = "File path",
                  subtype = 'FILE_PATH',
                  update=lambda a,b: materialFilenameUpdated(a,b,"thea_CoatingAnisotropyFilename")
                  )

Mat.thea_CoatingRotationFilename = bpy.props.StringProperty(
                  name = "Rotation texture",
                  default = "",
                  description = "File path",
                  subtype = 'FILE_PATH',
                  update=lambda a,b: materialFilenameUpdated(a,b,"thea_CoatingRotationFilename")
                  )              

Mat.thea_CoatingBumpFilename = bpy.props.StringProperty(
                  name = "Bump texture",
                  default = "",
                  description = "File path",
                  subtype = 'FILE_PATH',
                  update=lambda a,b: materialFilenameUpdated(a,b,"thea_CoatingBumpFilename")
                  )                                                                              

Mat.thea_SSS = bpy.props.BoolProperty(   
                name="SSS",
                description="Enable SSS material component",
                default= False,
                update=materialUpdated)

Mat.thea_SSSOrder = bpy.props.IntProperty(  
                min=0, 
                max=10,
                default=0,
                name="Order",
                description="Order",
                update=materialUpdated)                                          

Mat.thea_SSSWeight = bpy.props.FloatProperty(  
                min=0, 
                max=100,
                precision=2,
                default=100.0,
                name="Layer Weight",
                description="Layer Weight",
                update=materialUpdated)        

Mat.thea_SSSWeightFilename = bpy.props.StringProperty(
                  name = "Weight texture",
                  default = "",
                  description = "File path",
                  subtype = 'FILE_PATH',
                  update=lambda a,b: materialFilenameUpdated(a,b,"thea_SSSWeightFilename")
                  )          
                
Mat.thea_SSSReflectanceCol = bpy.props.FloatVectorProperty(
                min=0, max=1,  
                name="Reflectance Color", 
                description="Reflectance Color", 
                subtype="COLOR",
                update=materialUpdated)        

Mat.thea_SSSReflectanceFilename = bpy.props.StringProperty(
                  name = "Reflectance texture",
                  default = "",
                  description = "File path",
                  subtype = 'FILE_PATH',
                  update=lambda a,b: materialFilenameUpdated(a,b,"thea_SSSReflectanceFilename")
                  )
                
Mat.thea_SSSAbsorptionCol = bpy.props.FloatVectorProperty(
                min=0, max=1,  
                name="Absorption Color", 
                description="Absorption Color", 
                subtype="COLOR",
                update=materialUpdated)                        

Mat.thea_SSSAbsorption = bpy.props.FloatProperty(  
                min=0, 
                max=10000,
                precision=2,
                default=0.0,
                name="Absorption",
                description="Absorption",
                update=materialUpdated)                  

Mat.thea_SSSScatteringCol = bpy.props.FloatVectorProperty(
                min=0, max=1,  
                name="Scattering Color", 
                description="Scattering Color", 
                subtype="COLOR",
                update=materialUpdated)                        

Mat.thea_SSSScattering = bpy.props.FloatProperty(  
                min=0, 
                max=100000,
                precision=2,
                default=1000.0,
                name="Scattering",
                description="Scattering",
                update=materialUpdated)                  

Mat.thea_SSSAsymetry = bpy.props.FloatProperty(  
                min=0, 
                max=1,
                precision=2,
                default=0.0,
                name="Asymetry",
                description="Asymetry",
                update=materialUpdated)                                  

Mat.thea_SSSIOR = bpy.props.FloatProperty(  
                min=0, 
                max=1000,
                precision=3,
                default=1.5,
                name="Index of Refraction (n)",
                description="Index of Refraction (n)",
                update=materialUpdated)

Mat.thea_SSSTraceReflections = bpy.props.BoolProperty(   
                name="Trace Reflections",
                description="Trace Reflections",
                default= True,
                update=materialUpdated)                                          

Mat.thea_SSSTraceRefractions = bpy.props.BoolProperty(   
                name="Trace Refractions",
                description="Trace Refractions",
                default= True,
                update=materialUpdated)                                                          

Mat.thea_SSSStructureRoughness = bpy.props.FloatProperty(  
                min=0, 
                max=100,
                precision=2,
                default=0.0,
                name="Roughness (%)",
                description="Roughness (%)",
                subtype='PERCENTAGE',
                update=materialUpdated)

Mat.thea_SSSStructureRoughTrEn = bpy.props.BoolProperty(   
                name="Roughness Tr (%)",
                description="Roughness Tr (%)",
                default= False,
                update=materialUpdated)                                                          
                
                
Mat.thea_SSSStructureRoughnessTr = bpy.props.FloatProperty(  
                min=0, 
                max=100,
                precision=2,
                default=10.0,
                name="(%)",
                description="(%)",
                subtype='PERCENTAGE',
                update=materialUpdated)                
                
Mat.thea_SSSStructureAnisotropy = bpy.props.FloatProperty(  
                min=0, 
                max=100,
                precision=2,
                default=0.0,
                name="Anisotropy (%)",
                description="Anisotropy (%)",
                subtype='PERCENTAGE',
                update=materialUpdated)
                
Mat.thea_SSSStructureRotation = bpy.props.FloatProperty(  
                min=0, 
                max=100,
                precision=2,
                default=0.0,
                name="Rotation (deg)",
                description="Rotation (deg)",
                update=materialUpdated)                
                
Mat.thea_SSSStructureBump = bpy.props.FloatProperty(  
                min=0, 
                max=100,
                precision=2,
                default=100.0,
                name="Bump (%)",
                description="Bump (%)",
                subtype='PERCENTAGE',
                update=materialUpdated)                

Mat.thea_SSSBumpFilename = bpy.props.StringProperty(
                  name = "Bump texture",
                  default = "",
                  description = "File path",
                  subtype = 'FILE_PATH',
                  update=lambda a,b: materialFilenameUpdated(a,b,"thea_SSSBumpFilename")
                  )
    

Mat.thea_SSSRoughnessFilename = bpy.props.StringProperty(
                  name = "Roughness texture",
                  default = "",
                  description = "File path",
                  subtype = 'FILE_PATH',
                  update=lambda a,b: materialFilenameUpdated(a,b,"thea_SSSRoughnessFilename")
                  )

Mat.thea_SSSAnisotropyFilename = bpy.props.StringProperty(
                  name = "Anisotropy texture",
                  default = "",
                  description = "File path",
                  subtype = 'FILE_PATH',
                  update=lambda a,b: materialFilenameUpdated(a,b,"thea_SSSAnisotropyFilename")
                  )

Mat.thea_SSSRotationFilename = bpy.props.StringProperty(
                  name = "Rotation texture",
                  default = "",
                  description = "File path",
                  subtype = 'FILE_PATH',
                  update=lambda a,b: materialFilenameUpdated(a,b,"thea_SSSRotationFilename")
                  )

Mat.thea_SSSStructureNormal = bpy.props.BoolProperty(   
                name="Normal mapping",
                description="Enable normal mapping",
                default= False,
                update=materialUpdated)
                
Mat.thea_ThinFilm = bpy.props.BoolProperty(   
                name="ThinFilm Component",
                description="Enable ThinFilm material component",
                default= False,
                update=materialUpdated)

Mat.thea_ThinFilmOrder = bpy.props.IntProperty(  
                min=0, 
                max=10,
                default=0,
                name="Order",
                description="Order",
                update=materialUpdated)                                          

Mat.thea_ThinFilmWeight = bpy.props.FloatProperty(  
                min=0, 
                max=100,
                precision=2,
                default=100.0,
                name="Layer Weight",
                description="Layer Weight",
                update=materialUpdated)            

Mat.thea_ThinFilmWeightFilename = bpy.props.StringProperty(
                  name = "Weight texture",
                  default = "",
                  description = "File path",
                  subtype = 'FILE_PATH',
                  update=lambda a,b: materialFilenameUpdated(a,b,"thea_ThinFilmWeightFilename")
                  )      
                               
Mat.thea_ThinFilmTransmittanceCol = bpy.props.FloatVectorProperty(
                min=0, max=1,  
                name="Transmittance Color", 
                description="Transmittance Color", 
                subtype="COLOR",
                update=materialUpdated)                        

Mat.thea_ThinFilmIOR = bpy.props.FloatProperty(  
                min=0, 
                max=1000,
                precision=3,
                default=1.5,
                name="Index of Refraction (n)",
                description="Index of Refraction (n)",
                update=materialUpdated)

Mat.thea_ThinFilmInterference = bpy.props.BoolProperty(   
                name="Interference",
                description="Interference",
                default= False,
                update=materialUpdated)
                
Mat.thea_ThinFilmThickness = bpy.props.FloatProperty(  
                min=0, 
                max=20000,
                precision=3,
                default=500.0,
                name="Thickness (nm)",
                description="Thickness (nm)",
                update=materialUpdated)                                
                
Mat.thea_ThinFilmStructureBump = bpy.props.FloatProperty(  
                min=0, 
                max=100,
                precision=2,
                default=100.0,
                name="Bump (%)",
                description="Bump (%)",
                subtype='PERCENTAGE',
                update=materialUpdated)                

Mat.thea_ThinFilmStructureNormal = bpy.props.BoolProperty(   
                name="Normal mapping",
                description="Enable normal mapping",
                default= False,
                update=materialUpdated)                     

Mat.thea_StrandRoot = bpy.props.FloatProperty(  
                min=0, 
                max=100,
                precision=3,
                default=0.01,
                name="Root [m]",
                description="Strand root size[m]",
                update=materialUpdated)                                  

Mat.thea_StrandTip = bpy.props.FloatProperty(  
                min=0, 
                max=100,
                precision=3,
                default=0.001,
                name="Tip [m]",
                description="Strand tip size [m]",
                update=materialUpdated)                                  

Mat.thea_FastHairExport = bpy.props.BoolProperty(   
                name="Fast export",
                description="Enable fast hair export, but no UV mapping",
                default= False,
                update=materialUpdated)  

Mat.thea_ColoredHair = bpy.props.BoolProperty(   
                name="Colored Hair",
                description="Enable colored hair",
                default= False,
                update=materialUpdated)  

                
Obj.aperture = bpy.props.FloatProperty(  
                min=0, 
                max=1000,
                precision=2,
                default=5.6,
                name="Aperture",
                description="Aperture")

Obj.shutter_speed = bpy.props.FloatProperty(  
                min=0, 
                max=1000000,
                precision=0,
                default=500,
                name="Shutter speed",
                description="Shutter speed (1/s)")

Obj.autofocus = bpy.props.BoolProperty(   
                name="Autofocus",
                description="Autofocus",
                default= True)          

Obj.thea_projection = bpy.props.EnumProperty(   
                items=(("Perspective","Perspective","Perspective"),("Cylindrical","Cylindrical","Cylindrical"),("Spherical","Spherical", "Spherical"),("Parallel", "Parallel", "Parallel")),
                name="Projection",
                description="Projection",
                default="Perspective")

Obj.thea_zClippingNearDistance = bpy.props.FloatProperty(  
                min=0, 
                max=10000,
                precision=2,
                default=1,
                name="Z-Clipping Near Distance",
                description="Z-Clipping Near Distance")               

Obj.thea_zClippingFarDistance = bpy.props.FloatProperty(  
                min=0, 
                max=10000,
                precision=2,
                default=1000,
                name="Z-Clipping Far Distance",
                description="Z-Clipping Far Distance")

Obj.thea_zClippingNear = bpy.props.BoolProperty(   
                name="Z-Clipping Near",
                description="Z-Clipping Near",
                default= False)

Obj.thea_zClippingFar = bpy.props.BoolProperty(   
                name="Z-Clipping Far",
                description="Z-Clipping Far",
                default= False)

                
Obj.thEnabled = bpy.props.BoolProperty(   
                name="Enabled",
                description="Enable",
                default= True)                

Obj.thExportAnimation = bpy.props.BoolProperty(   
                name="Export Animation data of object",
                description="Export Animation data of object",
                default= False)

Obj.thShadowCaster = bpy.props.BoolProperty(   
                name="Shadow Caster",
                description="Shadow Caster",
                default= True)
                
Obj.thVisible = bpy.props.BoolProperty(   
                name="Visible",
                description="Visible",
                default= True)
                
Obj.thShadowTight = bpy.props.BoolProperty(   
                name="Shadow Tight",
                description="Shadow Tight",
                default= True)
                
Obj.thShadowReceiver = bpy.props.BoolProperty(   
                name="Shadow Receiver",
                description="Shadow Receiver",
                default= True)
                
Obj.thCausticsTransmitter = bpy.props.BoolProperty(   
                name="Caustics Transmitter",
                description="Caustics Transmitter",
                default= True)
                
Obj.thCausticsReceiver = bpy.props.BoolProperty(   
                name="Caustics Receiver",
                description="Caustics Receiver",
                default= True)
                
Obj.thNoRecalcNormals = bpy.props.BoolProperty(   
                name="Don't recalculate Normals while exporting",
                description="Don't recalculate Normals while exporting",
                default= False)          

def materialsList(scene, context):
    items = []
    items.append(("None","None","None"))
    if hasattr(bpy.data, "materials"):
        for mat in bpy.data.materials:
            items.append((mat.name, mat.name, mat.name))
    return items

Obj.thea_Container = bpy.props.EnumProperty(   
                 name="Materials List",
                 description="Materials List",
                 items=materialsList)          

def updateLUT(self, context):
    thea_globals.useLUT = getattr(context.scene, 'thea_useLUT')
    thea_globals.nameBasedLUT = getattr(context.scene, 'thea_nameBasedLUT')
    thea_globals.setConfig()
    print("thea_globals.useLUT: ", thea_globals.useLUT)
    bpy.ops.thea.refresh_diffuse_color()
                    
Scene.thea_useLUT = bpy.props.BoolProperty(   
                name="Use LUT",
                description="Use LUT file",
                #default= False,
                default=thea_globals.getUseLUT(),
                update=updateLUT)

Scene.thea_nameBasedLUT = bpy.props.BoolProperty(   
                name="Material name based LUT",
                description="Material name based LUT",
                default= thea_globals.getNameBasedLUT(),
                update=updateLUT)
                
Scene.thea_overwriteLUT = bpy.props.BoolProperty(   
                name="Overwrite LUT",
                description="Overwrite existing LUT file",
                default= False)

Scene.thea_LUTScanSubdirectories = bpy.props.BoolProperty(   
                name="Scan subdirectories for material files",
                description="Scan subdirectories for material files",
                default= True)                                
                
Scene.thea_materialsPath = bpy.props.StringProperty(   
                name="Thea materials path",
                description="Thea materials path",
                default= os.path.join(dataPath, "Materials"))                                

Scene.thea_lutMessage = bpy.props.StringProperty( 
                maxlen=40,
                default=" ",
                name="LUT",
                description="LUT")                

Scene.thea_SDKPort = bpy.props.IntProperty( 
                min=10000, 
                max=60000,
                default=30000,
                name="Port",
                description="Port")

Scene.thea_PreviewSDKPort = bpy.props.IntProperty( 
                min=10000, 
                max=60000,
                default=30001,
                name="Preview port",
                description="Preview port")

Scene.thea_RefreshDelay = bpy.props.FloatProperty(  
                min=0.01, 
                max=20,
                precision=1,
                default=0.1,
                name="Refresh delay",
                description="Refresh delay")

Scene.thea_StartTheaDelay = bpy.props.FloatProperty(  
                min=0.2, 
                max=5,
                precision=1,
                default=2,
                name="Delay after Thea start",
                description="Delay after Thea start")

Scene.thea_IRIdleTimeout = bpy.props.IntProperty( 
                min=10, 
                max=600,
                default=120,
                name="Idle timeout",
                description="How long to keep remote darkroom app running after rendering is finished")


Scene.thea_IRAlpha = bpy.props.FloatProperty(  
                min=0.0, 
                max=1,
                precision=1,
                default=1,
                name="IR preview alpha",
                description="IR preview alpha")

Scene.thea_IRMessage = bpy.props.StringProperty(
                  name = "Message",
                  default = "",
                  description = "IR Message",
                  )

Scene.thea_IRBlendAlpha = bpy.props.BoolProperty(   
                name="Blend 3D view to IR",
                description="Blend 3D view to IR",
                default= False)

updateTime = time.time()

def updateShowIR(self, context):
    global updateTime
    if (time.time()-updateTime) > 1:
        updateTime = time.time()
        port = getattr(context.scene, 'thea_SDKPort')            
        data = sendSocketMsg('localhost', port, b'version')        
        if getattr(context.scene, 'thea_IRShowTheaWindow'):
            context.scene.thea_DrawPreviewto3dView = False
            context.scene.thea_SavePreviewtoImage = False
            context.scene.thea_IRShowTheaWindow = True
            if data.find('v'):        
                message = b'message "./UI/Show" ' 
                data = sendSocketMsg('localhost', port, message)
        else:
            context.scene.thea_DrawPreviewto3dView = True
            context.scene.thea_SavePreviewtoImage = True
            context.scene.thea_IRShowTheaWindow = False
    updateTime = time.time()
    

   

        
def updateDrawIR(self, context):
    global updateTime
    if (time.time()-updateTime) > 1:
        updateTime = time.time()
        port = getattr(context.scene, 'thea_SDKPort')            
        data = sendSocketMsg('localhost', port, b'version')
        if getattr(context.scene, 'thea_DrawPreviewto3dView'):
            context.scene.thea_IRShowTheaWindow = False
            context.scene.thea_DrawPreviewto3dView = True
            context.scene.thea_SavePreviewtoImage = True
            if data.find('v'):        
                message = b'message "./UI/Hide"' 
                data = sendSocketMsg('localhost', port, message)
        else:
            #context.scene.thea_IRShowTheaWindow = True
            context.scene.thea_DrawPreviewto3dView = False
            #context.scene.thea_SavePreviewtoImage = False
    updateTime = time.time()
    
def updateSaveIR(self, context):
    global updateTime
    if (time.time()-updateTime) > 1:
        updateTime = time.time()
        port = getattr(context.scene, 'thea_SDKPort')            
        data = sendSocketMsg('localhost', port, b'version')
        if getattr(context.scene, 'thea_SavePreviewtoImage'):
            context.scene.thea_IRShowTheaWindow = False
            if data.find('v'):        
                message = b'message "./UI/Hide"' 
                data = sendSocketMsg('localhost', port, message)
        else:
            context.scene.thea_IRShowTheaWindow = True
            context.scene.thea_SavePreviewtoImage = False
            context.scene.thea_DrawPreviewto3dView = False
    updateTime = time.time()    
        
Scene.thea_IRShowTheaWindow = bpy.props.BoolProperty(   
                name="Show Thea window",
                description="Show Thea window when IR is running",
                default= True,
                update=updateShowIR)                    

Scene.thea_DrawPreviewto3dView = bpy.props.BoolProperty(   
                name="Draw to 3D view",
                description="Draw IR preview to 3dView",
                default= False,
                update=updateDrawIR)

Scene.thea_SavePreviewtoImage = bpy.props.BoolProperty(   
                name="Save IR to image",
                description="Save IR to image",
                default= False,
                update=updateSaveIR)

Scene.thea_IRFullUpdate = bpy.props.BoolProperty(   
                name="Full Auto refresh",
                description="Do full scene refresh after update",
                default= False)

Scene.thea_IRExportAnimation = bpy.props.BoolProperty(   
                name="Export Animation data of objects",
                description="Export Animation data of objects",
                default= False)

Scene.thea_IRDevice = bpy.props.EnumProperty(   
                items=(("GPU+CPU","GPU+CPU","GPU+CPU"),("GPU","GPU","GPU"),("CPU","CPU","CPU")),                
                name="Device",
                description="Device",
                default="GPU+CPU")

Scene.thea_IRResolution = bpy.props.EnumProperty(   
                items=(("Full 3D view size", "Full 3D view size", "Full 3D view size"),("Half size", "Half size", "Half size"),("Quarter size", "Quarter size", "Quarter size")),                
                name="Render resulution",
                description="Resoultion of IR render",
                default="Full 3D view size")

Scene.thea_IRAdvancedSettings = bpy.props.BoolProperty(   
                name="Advanced",
                description="Advanced",
                default= False)                    

Scene.thea_ir_running = bpy.props.BoolProperty(default=False)
Scene.thea_ir_update = bpy.props.BoolProperty(default=False)
Scene.thea_lastIRUpdate = bpy.props.IntProperty()

Part.thea_ApplyModifier = bpy.props.BoolProperty(   
                name="Apply modifier before export",
                description="Apply modifier before export",
                default= False)   

def materialComponentChanged(self, context):
    
    mat = context.material
#    mat.thea_MaterialComponent.items=getMaterialComponents()
#     if mat.thea_MaterialComponent == "Basic":
#         mat.thea_Basic = True
#         mat.thea_Glossy = False
#         mat.thea_SSS = False
#         mat.thea_ThinFilm = False
#     if mat.thea_MaterialComponent == "Glossy":
#         mat.thea_Basic = False
#         mat.thea_Glossy = True
#         mat.thea_SSS = False
#         mat.thea_ThinFilm = False
#     if mat.thea_MaterialComponent == "SSS":
#         mat.thea_Basic = False
#         mat.thea_Glossy = False
#         mat.thea_SSS = True
#         mat.thea_ThinFilm = False
#     if mat.thea_MaterialComponent == "ThinFilm":
#         mat.thea_Basic = False
#         mat.thea_Glossy = False
#         mat.thea_SSS = False        
#         mat.thea_ThinFilm = True
    
def getMaterialComponents(self, context):
    components = ("Basic", "Basic2", "Glossy", "Glossy2",  "SSS", "ThinFilm")
    items = []
    i=0    
    for component in components:
        #icon="RADIOBUT_OFF"
        icon="CHECKBOX_DEHLT"
        if hasattr(bpy.context, 'active_object'):
            if getattr(bpy.context.active_object.active_material, "thea_"+component):
                #icon="RADIOBUT_ON"
                icon="CHECKBOX_HLT"
        items.append((component, component, component, icon, i))
        i+=1
    return items
    
Mat.thea_MaterialComponent = bpy.props.EnumProperty(   
                #items=(("Basic","Basic","Basic", "RADIOBUT_OFF", 0),("Basic2","Basic2","Basic2", "RADIOBUT_OFF", 0),("Glossy","Glossy","Glossy", "RADIOBUT_OFF", 0),("Glossy2","Glossy2","Glossy2", "RADIOBUT_OFF", 0),("SSS","SSS","SSS", "RADIOBUT_OFF", 0),("ThinFilm","ThinFilm","ThinFilm", "RADIOBUT_OFF", 0)),
                items=getMaterialComponents,                
                name="Material",
                description="Material",
                #default="Basic",
                update=materialComponentChanged)

             

Scene.thea_showTools = bpy.props.BoolProperty(   
                name="Show",
                description="Show",
                default= False)

Scene.thea_showSetup = bpy.props.BoolProperty(   
                name="Show",
                description="Show",
                default= False)

Scene.thea_showRemoteSetup = bpy.props.BoolProperty(   
                name="Show",
                description="Show",
                default= False)

Scene.thea_showMerge = bpy.props.BoolProperty(   
                name="Show",
                description="Show",
                default= False)

Scene.thea_showChannels = bpy.props.BoolProperty(   
                name="Show",
                description="Show",
                default= False)

Scene.thea_enablePresets = bpy.props.BoolProperty(   
                name="Enable",
                description="Enable",
                default= False)                 

Lamp.thea_EmittanceUnit = bpy.props.EnumProperty(   
                items=(("Lumens","Lumens","Lumens"),("Candelas","Candelas","Candelas"),("Watts","Watts","Watts"),("W/sr","W/sr","W/sr"),("W/nm","W/nm","W/nm"),("W/nm/sr","W/nm/sr","W/nm/sr")),                
                name="Unit",
                description="Unit",
                default="Watts")

Lamp.thea_SunEmittanceUnit = bpy.props.EnumProperty(   
                items=(("Lumens","Lumens","Lumens"),("lm/m2","lm/m2","lm/m2"),("Candelas","Candelas","Candelas"),("cd/m2","cd/m2","cd/m2"),("Watts","Watts","Watts"),("W/m2","W/m2","W/m2"),("W/sr","W/sr","W/sr"),("W/sr/m2","W/sr/m2","W/sr/m2"),("W/nm","W/nm","W/nm"),("W/nm/m2","W/nm/m2","W/nm/m2"),("W/nm/sr","W/nm/sr","W/nm/sr"),("W/nm/sr/m2","W/nm/sr/m2","W/nm/sr/m2")),                
                name="Unit",
                description="Unit",
                default="W/nm/sr")

def lampFilenameUpdated(self, context, origin=""):    
    imgName = context.lamp.get(origin)    
    texName = "LAMP_"+origin
    lamp = context.lamp
    print("lampFilenameUpdated: ",self, context, origin, context.lamp.get(origin))
    exists = False
    try:
        if lamp.texture_slots[texName]:
            exists = True
            slot = lamp.texture_slots[texName]
            tex = slot.texture                       
    except:
        pass
            
    if exists:
        try:
            if imgName:
                img = bpy.data.images.load(imgName)
                tex.image = img
            else:
                print("removing texture: ", slot, tex)
                lamp.texture_slots[texName].texture = None
        except:
            pass
    else:
        img = bpy.data.images.load(imgName)
        tex = bpy.data.textures.new(name=texName, type='IMAGE') 
        tex.image = img
        tex.name = texName
        slot = lamp.texture_slots.add()
        slot.texture = tex

Lamp.thea_TextureFilename = bpy.props.StringProperty(
                  name = "Texture",
                  default = "",
                  description = "File path",
                  subtype = 'FILE_PATH',
                  update=lambda a,b: lampFilenameUpdated(a,b,"thea_TextureFilename")
                  )

Lamp.thea_IESFilename = bpy.props.StringProperty(
                  name = "IES file",
                  default = "",
                  description = "File path",
                  subtype = 'FILE_PATH')
                  
                  
Lamp.thea_enableLamp = bpy.props.BoolProperty(   
                name="Enable",
                description="Enable",
                default= True)

Lamp.thea_enableShadow = bpy.props.BoolProperty(   
                name="Shadow",
                description="Shadow",
                default= True)

Lamp.thea_enableSoftShadow = bpy.props.BoolProperty(   
                name="Soft Shadow",
                description="Soft Shadow",
                default= False)               

Lamp.thea_softRadius = bpy.props.FloatProperty(  
                min=0, 
                max=100000,
                precision=3,
                default=0.200,
                name="",
                description="Size")

Lamp.thea_radiusMultiplier = bpy.props.FloatProperty(  
                min=0, 
                max=100000,
                precision=3,
                default=1,
                name="",
                description="Radius Multiplier")

Lamp.thea_minRays = bpy.props.IntProperty(   
                min=0, 
                max=1000,
                default=8,
                name="Min Rays",
                description="Min Rays")

Lamp.thea_maxRays = bpy.props.IntProperty(   
                min=0, 
                max=1000,
                default=100,
                name="Max Rays",
                description="Max Rays")

Lamp.thea_globalPhotons = bpy.props.BoolProperty(   
                name="Global photons",
                description="Global photons",
                default= True)        

Lamp.thea_causticPhotons = bpy.props.BoolProperty(   
                name="Caustic photons",
                description="Caustic photons",
                default= True)         

Lamp.thea_bufferIndex = bpy.props.IntProperty(   
                min=0, 
                max=1000,
                default=0,
                name="Light Buffer Index",
                description="Light Buffer Index") 

Lamp.thea_EmittanceUnit = bpy.props.EnumProperty(   
                items=(("Lumens","Lumens","Lumens"),("Candelas","Candelas","Candelas"),("Watts","Watts","Watts"),("W/sr","W/sr","W/sr"),("W/nm","W/nm","W/nm"),("W/nm/sr","W/nm/sr","W/nm/sr")),                
                name="Unit",
                description="Unit",
                default="Watts")

Lamp.thea_EmittanceEfficacy = bpy.props.FloatProperty(  
                min=0, 
                max=1000000000,
                precision=3,
                default=20.0,
                name="Efficacy (lm/W)",
                description="Efficacy (lm/W)")
